# -*- coding: utf-8 -*-
import core_helper.helper_general as hg
hg.set_base_path()
#import lgb_model as l
import src.Prj_Core.core_helper.model.lgb_model as l
#import general as g
import src.Prj_Core.core_helper.model.general as g

import time
import core_helper.helper_plot as  hp

def get_neg_bagging_fraction_params(y_train,params):
    
    N_p = y_train[y_train==1].count()
    N_n = y_train[y_train==0].count()
    #T_minimo= 43000
    T_minimo = get_Total_min_to_train(N_p)
    print("T_minimo : ",T_minimo)
    print("N_p : ",N_p)
    print("N_n : ",N_n)
    
    alpha = (T_minimo-N_p)/N_n
    if params is None:
        params = l.get_default_params()
        
    params['pos_bagging_fraction'] = 1
    params['neg_bagging_fraction'] = alpha
    params['scale_pos_weight'] = (alpha*N_n)/N_p
    return params


def modelar(X_train,y_train=None,X_test=None,y_test=None,params=None,url=None):
    start = time.time()
  
    params = get_neg_bagging_fraction_params(y_train,params)

    model = l.lgb_model(X_train,y_train,X_test,y_test,params=params)
    if url is not None:
        g.save_model(model,url)
        g.save_json(params,url+"/params.json")
    
    predicted_probas = model.predict_proba(X_test) 
    
    return model , predicted_probas 



def modelar_rscv(X_train,y_train=None,X_test=None,y_test=None,score_rs=None,params=None,param_dist=None, n_iter=None,n_jobs=None,url=None):
    start = time.time()
    
    params = get_neg_bagging_fraction_params(y_train,params)  

    results, model , best_params = l.lgb_model_rscv(X_train,y_train,X_test,y_test,score_rs,params,param_dist, n_iter,n_jobs)
    g.save_model(model,url)
    results.to_excel(url+"/rscv_iteraciones.xlsx")    
    predicted_probas = model.predict_proba(X_test) 

    return  results, model , predicted_probas, params, best_params




def get_Total_min_to_train(N_p):
    T_minimo = 10000 
    
    while N_p>=T_minimo:
      T_minimo = T_minimo + 30000

    return T_minimo
    



# -*- coding: utf-8 -*-
"""
Created on Fri Apr  9 00:36:06 2021

@author: User
"""
import pandas as pd
import numpy as np
import src.Prj_Core.core_helper.helper_acces_db as hadb
import src.Prj_Core.core_helper.helper_clean as hc
import src.Prj_Core.core_helper.helper_general as hg

def agregar_sisfoh(df,df_sisfoh=None):    
    
    if df_sisfoh is None:
        df_sisfoh = hadb.get_sisfoh()  
    
    if 'NUMERO_DOCUMENTO_APOD' not in df.columns:
        msg = "ERROR: No existe la columnna NUMERO_DOCUMENTO_APOD en el DF proporcionado"
        raise Exception(msg)
        
    df['NUMERO_DOCUMENTO_APOD'] = df['NUMERO_DOCUMENTO_APOD'].str.replace('.0', '')
    df['NUMERO_DOCUMENTO_APOD']=df['NUMERO_DOCUMENTO_APOD'].apply(lambda x: '{0:0>8}'.format(x))
    df['NUMERO_DOCUMENTO_APOD'] = df['NUMERO_DOCUMENTO_APOD'].str.replace('00000nan', '00000000')     
    
    df = pd.merge(df, df_sisfoh, left_on=["NUMERO_DOCUMENTO_APOD"], right_on=["PERSONA_NRO_DOC"],  how='left')
    df = hc.fill_nan_with_nan_category_in_cls(df , ["SISFOH_CSE"])

    del df["PERSONA_NRO_DOC"]
    
    return df

# solo disponible 2019 y 2021 , B0 y F0
def agregar_shock_economico(df,df_se=None,anio=None,modalidad="EBR"):
    
    if df_se is None:
        df_se = hadb.get_shock_economico(anio,modalidad)
    
    print("hola")
    df = pd.merge(df, df_se, left_on="ID_PERSONA", right_on="ID_PERSONA",  how='inner')

    #df.fillna({'LOG_ING_T_MAS_1_IMP_DIST':0 }, inplace=True)

    return df

'''
dtypes_columns = {'COD_MOD': str,
                  'ANEXO':int,                
                  'COD_MOD_T_MENOS_1':str,
                  'ANEXO_T_MENOS_1':int,                  
                  'UBIGEO_NACIMIENTO_RENIEC':str,
                  'N_DOC':str,
                  'ID_GRADO':int,
                  'ID_PERSONA':int,#nurvo
                  'CODIGO_ESTUDIANTE':str,
                  'NUMERO_DOCUMENTO':str,
                  'NUMERO_DOCUMENTO_APOD':str,
                  'CODOOII':str
                  } 
url = hg.get_base_path()+"\\src\\Prj_Interrupcion_Estudios\\Prj_Desercion\\_02_Preparacion_Datos\\_02_Estructura_Base\\_data_\\nominal\\estructura_base_EBR_{}_{}_delta_1.csv"
url = url.format(5,2019)
df =pd.read_csv(url, dtype=dtypes_columns ,encoding="utf-8")


df = hadb.get_siagie_por_anio(2019,id_grado=5) 
print(df.shape)
df = agregar_shock_economico(df,anio=2019)
print(df.shape)
df_se = hadb.get_shock_economico(2019)
df_temp = df.head(100)
'''


import requests
import json
import sys
import os
import functools
import operator

def get_my_ip():
    url = 'https://api.myip.com'
    r = requests.get(url)
    dato = json.loads(r.text)
    return dato['ip']

def set_base_path():
    token = "\\src\\"
    test_string = os.path.abspath(os.curdir)
    PATH_BASE = test_string.partition(token)[0]
    separator = test_string.partition(token)[1]
    if separator==token:
        sys.path.append(PATH_BASE)
        print("seteando PATH_BASE : ",PATH_BASE ) 


def get_base_path():
    token = "\\src\\"
    curdir_path = os.path.abspath(os.curdir)
    PATH_BASE = curdir_path.partition(token)[0]
    separator = curdir_path.partition(token)[1]
    if separator==token:
        return  PATH_BASE
    else:
        return curdir_path
    
def flat_list(List_2D):   
    List_flat = functools.reduce(operator.iconcat, List_2D, [])    
    return List_flat

def validar_directorio(path):
    if os.path.isdir(path)==False:
        os.makedirs(path)


def main():
    print(get_my_ip())


if __name__ == "__main__":
    main()


# -*- coding: utf-8 -*-
"""
Created on Mon Jun 28 08:51:04 2021

@author: User
"""

# -*- coding: utf-8 -*-
import numpy as np
import pandas as pd
from urllib.request import urlopen

from sklearn.model_selection import train_test_split


#import core_helper.helper_general as hg
import src.Prj_Core.core_helper.helper_general as hg
hg.set_base_path()


#import core_helper.helper_acces_db as hadb
import src.Prj_Core.core_helper.helper_acces_db as hadb

import src.Prj_Core.core_helper.helper_feature_selection as hfs

#import core_helper.helper_transformers as ht
import src.Prj_Core.core_helper.helper_transformers as ht

import src.Prj_Core.core_helper.helper_clean as hc
import src.Prj_Core.core_helper.helper_classification_model as hcm

import src.Prj_Core.core_helper.helper_dataframe as hd
import src.Prj_Core.core_helper.helper_siagie_kpi as hsk
import src.Prj_Core.core_helper.helper_terceros_kpi as htk

from sklearn.metrics import classification_report,average_precision_score
import pickle
import matplotlib.pyplot as plt
import seaborn as sns

from scipy import stats 
from sklearn.preprocessing import RobustScaler

def get_df_procesado_temp(anio,col_name_y,key_grupo_grados, grupo_grados,macro_region, 
                     feature_selection=False,df_id_persona=None,anio_serv=None):
    
    
    dtypes_columns = {'COD_MOD': str,
                      'ANEXO':int,                    
                      'COD_MOD_T_MENOS_N':str,
                      'ANEXO_T_MENOS_N':int,                      
                      'UBIGEO_NACIMIENTO_RENIEC':str,
                      'N_DOC':str,
                      'ID_GRADO':int,
                      'ID_PERSONA':int,#nurvo
                      'CODIGO_ESTUDIANTE':str,
                      'NUMERO_DOCUMENTO':str,
                      'NUMERO_DOCUMENTO_APOD':str,
                      'CODOOII':str
                      }   


    
    df_servicios = hadb.get_df_servicios(macro_region=macro_region,anio=anio_serv)    

    list_pd = []
    #5,6,7,8
    for ID_GRADO in grupo_grados:
        url = hg.get_base_path()+"\\src\\Prj_Interrupcion_Estudios\\Prj_Desercion\\_02_Preparacion_Datos\\_02_Estructura_Base\\_data_\\nominal\\estructura_base_EBR_{}_{}_delta_1_temp.csv"
        url = url.format(ID_GRADO,anio)
        df =pd.read_csv(url, dtype=dtypes_columns ,encoding="utf-8")
        df['STR_ID_GRADO'] = "GRADO_" + str(ID_GRADO)      
        df = pd.merge(df,df_servicios, left_on=["COD_MOD","ANEXO"], right_on = ["COD_MOD","ANEXO"] ,how="inner")
        if df_id_persona is not None:
            df = pd.merge(df,df_id_persona[["ID_PERSONA"]], left_on="ID_PERSONA", right_on = "ID_PERSONA",how="inner")        
        list_pd.append(df)
        
    df_reg = pd.concat(list_pd)
    print("df_reg.shape", df_reg.shape)
    anio_notas = anio-1   
    
    ''' 
    cls_json = {}
    cls_json['SITUACION_FINAL']=["APROBADO","DESAPROBADO"]
    cls_json['SF_RECUPERACION']=["APROBADO","DESAPROBADO"]
    cls_json['SITUACION_MATRICULA']=["PROMOVIDO","REPITE","INGRESANTE","REENTRANTE"]
    cls_json['JUNTOS']="dummy"
    
    df_reg = hsk.generar_kpis_historicos(df_reg,anio_df=anio,anio_h=anio-2,cls_json=cls_json,t_anios=4)      
    df_reg = hsk.agregar_notas(df_reg,anio,anio_notas, cls_group=["ZSCORE"])    
    df_reg = htk.agregar_sisfoh(df_reg)       
    df_reg = hsk.generar_kpis_desercion(df_reg,anio_df=anio, anio_h=anio-2 ,t_anios=4)  
      
    df_reg =  htk.agregar_shock_economico(df_reg,anio=anio)  
    
    df_reg = hsk.generar_kpis_traslado(df_reg,anio_df=anio,anio_h=anio-1,t_anios=5)
    df_reg = hsk.generar_kpis_traslado_a_publico(df_reg,anio_df=anio,anio_h=anio-1,t_anios=5)
    '''
    if key_grupo_grados=="6 prim":
        df_reg = hsk.agregar_distancia_prim_sec(df_reg)
        df_reg = hc.fill_nan_with_nan_category_in_cls(df_reg , ["GRUPO_DISTANCIA"])
    
    
    
    df_reg = hc.fill_nan_with_nan_category_in_cls(df_reg , ["SITUACION_MATRICULA_T_MENOS_N",
                                                            "SF_RECUPERACION_T_MENOS_N",
                                                            "PARENTESCO","DSC_DISCAPACIDAD",
                                                            "SEXO_APOD", "SITUACION_FINAL_T_MENOS_N",                                                        
                                                            "SITUACION_MATRICULA_T",
                                                            "TIENE_CERTIFICADO_DISCAPACIDAD",
                                                            "PADRE_VIVE","MADRE_VIVE"])
    
    df_reg = hc.trim_category_cls(df_reg)
    
    df_reg = hsk.formatear_columnas_siaguie(df_reg)
    
   
    
    #df_reg['EDAD_EN_DIAS_T']  = df_reg['EDAD_EN_DIAS_T'].round()
   
   
    columns_too_much_nan_and_categories = ["JUSTIFICACION_RETIRO_T_MENOS_N","DSC_PAIS","DSC_LENGUA"]
    columns_to_drop  = ['ID_PERSONA','COD_MOD','ANEXO','COD_MOD_T_MENOS_N','ANEXO_T_MENOS_N','ID_GRADO_T_MENOS_N',
                        'NUMERO_DOCUMENTO_APOD','N_DOC','NIVEL_INSTRUCCION_APOD'] 
    #cat_muy_largas = ['D_REGION']
    
    columns_to_drop_all = columns_too_much_nan_and_categories+columns_to_drop 
    
    ID_P_T = df_reg['ID_PERSONA']
    
    if col_name_y is not None:
        y = df_reg[col_name_y]
        columns_to_drop_all.append(col_name_y)
    else:    
        y = None
        
        
    X = df_reg.drop(columns = columns_to_drop_all) 

    ct = ht.CatTransformer(pp = "lb",console=True) #lb le
    ct.fit(X)
    X_t = ct.transform(X)
    #print(X_t.columns)
    if feature_selection:
        X_t = hfs.drop_cls_unique_value(X_t)
        X_t = hfs.drop_corr_columns(X_t)
        X_t = hfs.drop_nan_columns(X_t)
    
    return ID_P_T, X_t , y



import pandas as pd
from io import StringIO
from openpyxl.formatting.rule import ColorScaleRule
from openpyxl.styles import Alignment, Font, NamedStyle
from openpyxl.utils import get_column_letter


df = pd.read_csv(StringIO("""\
               alpha      beta     gamma
2000-01-01 -0.173215  0.119209 -1.044236
2000-01-02 -0.861849 -2.104569 -0.494929
2000-01-03  1.071804  0.721555 -0.706771
2000-01-04 -1.039575  0.271860 -0.424972
2000-01-05  0.567020  0.276232 -1.087401
2000-01-06 -0.673690  0.113648 -1.478427
2000-01-07  0.524988  0.404705  0.577046
2000-01-08 -1.715002 -1.039268 -0.370647
"""), sep="\s+", parse_dates=True)

output_filename = "salida.xlsx"


def no_customization(sheet_name='No customization'):
    df.to_excel(output_filename, sheet_name)


def little_customization(sheet_name='Little customization'):
    # Python context manager (with ... as ..:) automatically closes resource
    # when block is left, even under error conditions/cancellation/...
    with pd.ExcelWriter(
            output_filename,
            mode='a',  # append to existing file; default='w' (overwrite)
            engine='openpyxl') as xlsx:
        df.to_excel(xlsx, sheet_name)
        
        # set column width
        xlsx.sheets[sheet_name].column_dimensions['A'].width = 21


def full_customization(sheet_name='Full customization'):
    # Unpacking arguments using '**' for dictionary of keyword arguments
    # see: https://docs.python.org/3/tutorial/controlflow.html#more-on-defining-functions
    writer_args = {
        'path': output_filename,
        'mode': 'a',
        'engine': 'openpyxl'}
        
    # https://openpyxl.readthedocs.io/en/stable/formatting.html#colorscale
    percentile_rule = ColorScaleRule(
        start_type='percentile', start_value=10, start_color='ffaaaa',  # red-ish
        mid_type='num', mid_value=0, mid_color='ffffff',  # value zero==white
        end_type='percentile', end_value=90, end_color='aaffaa')  # green-ish
        
    # create a custom named style for the index
    index_style = NamedStyle(
        name="Index Style",
        number_format='YYYY-MM-DD, DDD',
        font=Font(color='999999', italic=True),
        alignment=Alignment(horizontal='left'))
    
    with pd.ExcelWriter(**writer_args) as xlsx:
        df.to_excel(xlsx, sheet_name)
        
        # worksheets that have been created with this ExcelWriter can be accessed
        # by openpyxl using its API. `ws` is now a openpyxl Worksheet object
        ws = xlsx.sheets[sheet_name]
        
        # cell ranges
        title_row = '1'
        value_cells = 'B2:{col}{row}'.format(
            col=get_column_letter(ws.max_column),
            row=ws.max_row)
        index_column = 'A'
        
        ws.column_dimensions[index_column].width = 21

        # which offers, among other things, a conditional_formatting facility
        ws.conditional_formatting.add(value_cells, percentile_rule)
        
        # for general styling, one has to iterate over all cells individually
        for row in ws[value_cells]:
            for cell in row:
                cell.number_format = '0.00'
        
        # builtin or named styles can be applied by using the object or their name
        # https://openpyxl.readthedocs.io/en/stable/styles.html#using-builtin-styles
        for cell in ws[index_column]:
            cell.style = index_style
        
        # style header line last, so that headline style wins in cell A1
        for cell in ws[title_row]:
            cell.style = 'Headline 2'
            

no_customization()
little_customization()
full_customization()

#################################### 1.FUSION ######################################



########################################################################################
'''
#ID_P_T, X_t , y = get_df_procesado_temp(2019,'DESERCION_2019_2020',"6 prim",[9],"oriente",True,None,2019)
ID_P_T, X_t , y = get_df_procesado(2019,'DESERCION_2019_2020',"6 prim",[9],"oriente",True,None,2019)
test_size = hcm.get_test_size(X_t)
X_train, X_test, y_train, y_test= train_test_split(X_t, y, test_size=test_size,stratify=y,random_state=42)
model_name = "neg_bagging_fraction__lgb_model"
model , predicted_probas  , KPIs = hcm.modelar_clasificacion_binaria(model_name, X_train,y_train,X_test,y_test)
'''
#########################################################################################
'''
ID_P_T, X_t , y = get_df_procesado(2019,'DESERCION_2019_2020',"6 prim",[9],"oriente",True,None,2019)
X_t['randNumCol'] = np.random.normal(3, 2.5, X_t.shape[0])
print(hd.get_bool_columns(X_t))
inputs_num = list(set(X_t.columns.tolist())-set(hd.get_bool_columns(X_t)))


data_ca = {'Variable' : inputs_num,
       'Coeficiente_Asim' : stats.skew(X_t[inputs_num])}
asim = pd.DataFrame(data_ca)


inputs_num_sel = np.array(asim.iloc[:,0][asim['Coeficiente_Asim']<=5])
inputs_num_sel

res_data_num = X_t[inputs_num].describe().transpose()
res_data_num['cv'] = res_data_num.iloc[:,2] / res_data_num.iloc[:,1] * 100
res_data_num

from sklearn.preprocessing import RobustScaler
transformer = RobustScaler().fit(X_t)
data_inputs_s = transformer.transform(X_t)
X_t = pd.DataFrame(data_inputs_s,columns=X_t.columns.tolist())

sns.kdeplot(X_t['randNumCol'],shade = True,vertical = False,cumulative = False,color = "#BB0000")
plt.show()

test_size = hcm.get_test_size(X_t)
X_train, X_test, y_train, y_test= train_test_split(X_t, y, test_size=test_size,stratify=y,random_state=42)
       

model_name = "neg_bagging_fraction__lgb_model"
model , predicted_probas  , KPIs = hcm.modelar_clasificacion_binaria(model_name, X_train,y_train,X_test,y_test)


specific_url_model = "temp/model.sav"
model = pickle.load( open( specific_url_model, "rb" ) )

y_pred2 ,y_prob2_cb,  predicted_probas2 = hcm.predecir_clasificacion_binaria(model,X_test)

'''






  


# Data handling and processing
import pandas as pd
import numpy as np
from imblearn.over_sampling import SMOTE
from imblearn.over_sampling import ADASYN
from imblearn.under_sampling import NearMiss
from imblearn.over_sampling import SMOTENC
from imblearn.pipeline import make_pipeline as imbalanced_make_pipeline
import random
#import statsmodels.api as sm
import matplotlib.patches as mpatches
import time
#import mpld3
from scipy.stats import randint as sp_randint
from scipy.stats import uniform as sp_uniform

from sklearn.manifold import TSNE
from sklearn.decomposition import PCA, TruncatedSVD

from sklearn.preprocessing import StandardScaler, RobustScaler

from sklearn.model_selection import train_test_split, RandomizedSearchCV, learning_curve,cross_validate
from unidecode import unidecode
# Data visualisation & images
import matplotlib.pyplot as plt
import seaborn as sns
import sys
# Pipeline and machine learning algorithms
from sklearn.preprocessing import LabelEncoder,label_binarize
from sklearn import preprocessing, decomposition, svm #, cross_validation
from sklearn.tree import DecisionTreeClassifier
from sklearn.discriminant_analysis import LinearDiscriminantAnalysis
from sklearn.linear_model import LogisticRegression
from sklearn.neighbors import KNeighborsClassifier
from sklearn.neural_network import MLPClassifier
from sklearn.svm import SVC

from sklearn.ensemble import RandomForestClassifier, AdaBoostClassifier, GradientBoostingClassifier, ExtraTreesClassifier, VotingClassifier

from sklearn.linear_model import LinearRegression
from sklearn.linear_model import LogisticRegression
from sklearn.naive_bayes import GaussianNB
from sklearn.neighbors import NearestNeighbors
from sklearn.feature_selection import SelectFromModel

from sklearn.metrics import *
#roc_auc_score ,f1_score,  accuracy_score,r2_score,mean_squared_error,neg_mean_absolute_error,
#neg_mean_squared_log_error,neg_median_absolute_error
    
from sklearn.externals import joblib
import random

import matplotlib.pylab as pl
import pandas as pd
import csv
import pyodbc

#from imblearn.over_sampling import SMOTE

import json

# Model fine-tuning and evaluation
from sklearn.model_selection import StratifiedKFold, cross_val_score
from sklearn.model_selection import GridSearchCV
from sklearn.model_selection import cross_val_predict

from xgboost import plot_importance
from xgboost import XGBClassifier

import lightgbm as lgb
import catboost as cb

import pickle
import os

import timeit    
import shap

from boruta import BorutaPy


from sklearn.metrics import roc_curve, precision_recall_curve, auc, make_scorer, recall_score, accuracy_score, precision_score, confusion_matrix, roc_auc_score

from scipy import optimize as op_sc




shap.initjs()

# def feature_sel_shap( X_train, y_train, X_val , y_val, min_shap = 0 , objective='binary' , metric_lgb='f1',num_class=2):
#     # print("---------aaa--------")
#     # X_train.loc[(X_train['NIVEL_INSTRUCCION_APOD_2017']=='NAN' ) , 'NIVEL_INSTRUCCION_APOD_2017'] = np.nan
#     # X_val.loc[(X_val['NIVEL_INSTRUCCION_APOD_2017']=='NAN' ) , 'NIVEL_INSTRUCCION_APOD_2017'] = np.nan

#     # X_train.loc[(X_train['UBIGEO_NACIMIENTO_RENIEC_2017']=='000000' ) , 'UBIGEO_NACIMIENTO_RENIEC_2017'] = np.nan
#     # X_val.loc[(X_val['UBIGEO_NACIMIENTO_RENIEC_2017']=='000000' ) , 'UBIGEO_NACIMIENTO_RENIEC_2017'] = np.nan

#     print("inicio del proceso: feature_sel_shap con  min_shap:"+str(min_shap))
#     train_cols = X_train.columns.tolist()

#     _cols = X_train.columns
#     _num_cols = X_train._get_numeric_data().columns
#     _categorical_columns_name = list(set(_cols) - set(_num_cols))


#     train_data = lgb.Dataset(X_train, label=y_train.astype(int), 
#                             feature_name=train_cols)
#     test_data = lgb.Dataset(X_val, label=y_val.astype(int), 
#                             feature_name=train_cols, reference=train_data)

#     # LGB parameters:
#     params = {'learning_rate': 0.05,
#             'boosting': 'gbdt', 
#             'objective': objective,
#             'num_leaves': 2000,
#             'min_data_in_leaf': 200,
#             'max_bin': 200,
#             'max_depth': 16,
#             'seed': 2018,
#             'nthread': 10,}

#     if(objective=='multiclass'):
#         params['num_class'] = num_class
#         params['metric'] = metric_lgb
    
#     print("entrenando para optener las variables mas importantes")
#     # LGB training:
#     lgb_model = lgb.train(params, train_data, 
#                         categorical_feature=_categorical_columns_name,
#                         num_boost_round=1000, 
#                         valid_sets=(test_data,), 
#                         valid_names=('valid',), 
#                         verbose_eval=False, 
#                         early_stopping_rounds=20)


#     # DF, based on which importance is checked
#     X_importance = X_val
#     print("recuperando las variablwa impoetantes del modelo")
#     # Explain model predictions using shap library:
#     explainer = shap.TreeExplainer(lgb_model)
#     print("TreeExplainer...done")
#     shap_values = explainer.shap_values(X_importance)
#     print("shap_values...done")

#     shap_sum = np.abs(shap_values).mean(axis=0)
#     print("shap_sum...done")
#     importance_df = pd.DataFrame([X_importance.columns.tolist(), shap_sum.tolist()]).T
#     importance_df.columns = ['column_name', 'shap_importance']
#     importance_df = importance_df.sort_values('shap_importance', ascending=False)
   

#     return shap_values,X_importance,importance_df



def hf_build_model(split_config=None,model_config=None):  

    min_shap = model_config.get('min_shap')  
    #score = model_config.get('score')   #scoring = {'roc_auc': 'f1','recall': 'precision'}
    
    metric_lgb = model_config.get('metric_lgb') 
    score_rs = model_config.get('score_rs') 
    
    objective = model_config.get('objective')
    best_params = model_config.get('best_params')
    num_class = model_config.get('num_class')
    
    if(objective is None):
        objective = 'binary'
    
    n_splits = 10
    kfold = StratifiedKFold(n_splits=n_splits, shuffle=True)

    #model_validaded = model = X_train= X_val = X_test = y_train = y_val = y_test = X_train_full =y_train_full = None
    X_train, X_val , X_test , y_train , y_val , y_test = hf_train_test_split(**split_config)

#     shap_values,X_importance, importance_df = feature_sel_shap(X_train, y_train, X_val , y_val 
#                                                       , min_shap,objective=objective , metric_lgb=metric_lgb,num_class=num_class )

#    return shap_values, X_importance, importance_df
    if(model_config.get('importance') is None or model_config.get('importance')=='shap'):
        print("0) Obteniendo variables importantes")
        zero_features, feature_importances = feature_sel_shap(X_train, y_train, X_val , y_val 
                                                              , min_shap,objective=objective , metric_lgb=metric_lgb,num_class=num_class )
        print("fin del proceso : 0")
    elif(model_config.get('importance')=='boruta'):
        zero_features, feature_importances = feature_sel_boruta(X_train, y_train)
    #zero_features, feature_importances = self.identify_zero_importance_features(X_train, y_train, score ,iterations=n_splits)

    X_train_full =  pd.concat(objs=[X_train, X_val ], axis=0).reset_index(drop=True)
    y_train_full =  np.concatenate([y_train,y_val])
    X_train = X_train.drop(columns = zero_features)
    X_val = X_val.drop(columns = zero_features)   
    X_test = X_test.drop(columns = zero_features) 
    X_train_full = X_train_full.drop(columns = zero_features)  

    start_time = timeit.default_timer()
    
    if(best_params is not None):
        print("Omitiendo calculo de hypermarametros")
        bp = best_params
    else:
        print("1) CALCULANDO LOS MEJORES HYPERPARAMETROS  ------")
        bm, bp = get_hyperparametros(X_train,y_train,X_val,y_val,
                                     score_rs=score_rs,metric_lgb=metric_lgb,_kfold=kfold,objective=objective,num_class=num_class)
    print(bp)
    #     #best_param = {'scale_pos_weight': 3, 'objective': 'binary', 'n_estimators': 450, 'max_depth': -1, 'learning_rate': 0.015}
    print("2) ENTRENANDO EL MODELO  ------")
    model_validaded = buildModel_LGBMClassifier(X_train,y_train,X_val,y_val, bp,metric_lgb=metric_lgb)
    # model_validaded_smote = self.buildModel_LGBMClassifier(X_train,y_train,X_val,y_val, best_param, smote = True)
    # print("---- TESTEANDO EL MODELO 2 smote ------") 
    # self.run_clasification(X_test,y_test,model_validaded_smote)

    print("3) TESTEANDO EL MODELO  ------") 
    run_clasification(X_test,y_test,model_validaded,score_rs=score_rs,objective=objective)

    print("3) RETORNANDO EL CLASIFICADOR VALIDO  ------") 
    print("Columnas del modelo: ",X_test.columns)
    return model_validaded,zero_features


def buildModel_LGBMClassifier(X_train,y_train,X_val,y_val, best_param,smote=False, metric_lgb='f1'):


    lgb_model = lgb.LGBMClassifier(**best_param)

    fit_params={
        'eval_metric':metric_lgb,
        'eval_set':[(X_val, y_val)], 
        'eval_names':['valid','train'],
        'early_stopping_rounds':10, 
        'feature_name': 'auto', # that's actually the default
        'categorical_feature': 'auto', # that's actually the default
        'verbose':False
    }


    if(smote):
        _cols = X_train.columns
        _num_cols = X_train._get_numeric_data().columns
        _categorical_columns_name = list(set(_cols) - set(_num_cols))
        _categorical_columns_index = [X_train.columns.get_loc(c) for c in _categorical_columns_name if c in X_train]

        print("categorica ..............")
        print(_categorical_columns_name)

        print("_num_cols ..............")
        print(_num_cols)

        smote_nc = SMOTENC(categorical_features=_categorical_columns_index, random_state=0)
        X_train_smote, y_train_smote = smote_nc.fit_resample(X_train,y_train)
        X_imputed_df = pd.DataFrame(X_train_smote, columns = X_train.columns)
        print("--------- imputed categorical ------")
        for c in _categorical_columns_name:                
            X_imputed_df[c] = X_imputed_df[c].astype('category') 

        for c in _num_cols:                
            X_imputed_df[c] = X_imputed_df[c].astype('float') 


        lgb_model.fit(X_imputed_df,y_train_smote,**fit_params)
    else:

        #lgb_model.fit(X_train, y_train,**fit_params)
        lgb_model.fit(X_train, y_train)



    return lgb_model



def get_hyperparametros(_X_train,_y_train,X_val,y_val, score_rs ='f1', metric_lgb='f1',_kfold=None , doSMOTE=False, gs=True,objective='binary',num_class=2):

    print("Optimizando del score_rs :",score_rs)

    gb_param_grid = {'learning_rate':[0.015,0.012,0.025,0.01,0.005],
                    #'num_leaves':[25,28,31,38,45,50,60,70,80],
                    #'metric': metric_lgb,
                    'max_depth': [-1],
                    'objective' : [objective],
                    'n_estimators':[150,160,180,170,200,250,300,350,400,450,500,550,600,650,700,750,800],#,
                    'scale_pos_weight':[2,3,4,5,6,7,8,9,10,11,12,13,14,15,16] , 


                    # 'colsample_bytree' : [0.65, 0.66,0.7,0.8,0.85,0.9,1.],
                    # 'subsample' : [0.5,0.6,0.7,0.75,0.80,0.85,1.],
                    # 'reg_alpha' : [0,0.25,0.5,0.8,1,1.2],
                    # 'reg_lambda' : [0,0.25,0.5,0.8,1,1.2,1.4]   

                    # 'min_child_samples': sp_randint(100, 500), 
                    # 'min_child_weight': [1e-5, 1e-3, 1e-2, 1e-1, 1, 1e1, 1e2, 1e3, 1e4],
                    # 'subsample': sp_uniform(loc=0.2, scale=0.8), 
                    # 'colsample_bytree': sp_uniform(loc=0.4, scale=0.6),
                    # 'reg_alpha': [0, 1e-1, 1, 2, 5, 7, 10, 50, 100],
                    # 'reg_lambda': [0, 1e-1, 1, 5, 10, 20, 50, 100]

                    }

#    if(objective=='multiclass'):
#         gb_param_grid['num_class'] = num_class
#         gb_param_grid['metric'] = metric_lgb
    # fit_params={
    #     'feature_name': 'auto', # that's actually the default
    #     'categorical_feature': 'auto' # that's actually the default
    # }

    fit_params={
                        "early_stopping_rounds":10, 
                        "eval_metric" : metric_lgb, 
                        "eval_set" : [(X_val,y_val)],
                        'eval_names': ['valid','train'],
                        'verbose': False,
                        'feature_name': 'auto', # that's actually the default
                        'categorical_feature': 'auto' # that's actually the default
                    }

    print("params to iterate : ",)
    model = lgb.LGBMClassifier()
    LGB_best =  LGB_best_param = LGB_SMOTE_best = gsLGB = gsLGB_SMOTE = None
    #

    if(gs): 
        print("...Optimizando hyperparametros y generando modelo 80 sm")
        #gsLGB = GridSearchCV(estimator = model ,param_grid = gb_param_grid,  scoring=score, n_jobs= 5,  cv=_kfold)  
        #score ['f1','precision']
        #score = {'F1': 'f1', 'PRECISION': 'precision'}
        gsLGB = RandomizedSearchCV(model, param_distributions=gb_param_grid, 
                                   scoring=score_rs ,  n_iter=40, cv=_kfold,n_jobs= 13, iid=False)
        #gsLGB.fit(_X_train,_y_train,**fit_params)
        
        if(objective=='multiclass'):
            __y_train = label_binarize(_y_train, classes=[0, 1, 2, 3])
            gsLGB.fit(_X_train,__y_train)
        else:
            gsLGB.fit(_X_train,_y_train)

        # _cols = _X_train.columns
        # _num_cols = _X_train._get_numeric_data().columns
        # _categorical_columns_name = list(set(_cols) - set(_num_cols))
        # _categorical_columns_index = [_X_train.columns.get_loc(c) for c in _categorical_columns_name if c in _X_train]

        # smote_nc = SMOTENC(categorical_features=_categorical_columns_index, random_state=0)
        # X_train_smote, y_train_smote = smote_nc.fit_resample(_X_train,_y_train)
        # X_imputed_df = pd.DataFrame(X_train_smote, columns = _X_train.columns)


        # for c in _categorical_columns_name:                
        #     X_imputed_df[c] = X_imputed_df[c].astype('category') 

        # for c in _num_cols:                
        #     X_imputed_df[c] = X_imputed_df[c].astype('float') 

        # gsLGB.fit(X_imputed_df,y_train_smote,**fit_params)


        LGB_best = gsLGB.best_estimator_ 
        print(LGB_best)
        print('Best params for {}'.format(score_rs))
        print(gsLGB.best_params_)
        LGB_best_param = gsLGB.best_params_
    else:
        print("...generando modelo")
        model.fit(_X_train,_y_train) 
        LGB_best = model

    if(doSMOTE):            
        #param = Nm/Nrm
        gsLGB_SMOTE = GridSearchCV(estimator = model ,param_grid = gb_param_grid,  scoring=score, n_jobs= 10,  cv=_kfold)
        smt = SMOTE()                 
        X_train_smote, y_train_smote = smt.fit_sample(_X_train , _y_train)
        gsLGB_SMOTE.fit(X_train_smote,y_train_smote)
        LGB_SMOTE_best = gsLGB_SMOTE.best_estimator_
        print('Best params for {}'.format(score_rs))
        print(gsLGB_SMOTE.best_params_)

        return LGB_best, LGB_SMOTE_best
    else:
        return LGB_best , LGB_best_param



    



def feature_sel_shap( X_train, y_train, X_val , y_val, min_shap = 0 , objective='binary' , metric_lgb='f1',num_class=2):
    # print("---------aaa--------")
    # X_train.loc[(X_train['NIVEL_INSTRUCCION_APOD_2017']=='NAN' ) , 'NIVEL_INSTRUCCION_APOD_2017'] = np.nan
    # X_val.loc[(X_val['NIVEL_INSTRUCCION_APOD_2017']=='NAN' ) , 'NIVEL_INSTRUCCION_APOD_2017'] = np.nan

    # X_train.loc[(X_train['UBIGEO_NACIMIENTO_RENIEC_2017']=='000000' ) , 'UBIGEO_NACIMIENTO_RENIEC_2017'] = np.nan
    # X_val.loc[(X_val['UBIGEO_NACIMIENTO_RENIEC_2017']=='000000' ) , 'UBIGEO_NACIMIENTO_RENIEC_2017'] = np.nan

    print("inicio del proceso: feature_sel_shap con  min_shap:"+str(min_shap))
    train_cols = X_train.columns.tolist()

    _cols = X_train.columns
    _num_cols = X_train._get_numeric_data().columns
    _categorical_columns_name = list(set(_cols) - set(_num_cols))


    train_data = lgb.Dataset(X_train, label=y_train.astype(int), 
                            feature_name=train_cols)
    test_data = lgb.Dataset(X_val, label=y_val.astype(int), 
                            feature_name=train_cols, reference=train_data)

    # LGB parameters:
    params = {'learning_rate': 0.05,
            'boosting': 'gbdt', 
            'objective': objective,
            'num_leaves': 2000,
            'min_data_in_leaf': 200,
            'max_bin': 200,
            'max_depth': 16,
            'seed': 2018,
            'nthread': 10,}

    if(objective=='multiclass'):
        params['num_class'] = num_class
        params['metric'] = metric_lgb
    
    print("entrenando para optener las variables mas importantes")
    # LGB training:
    lgb_model = lgb.train(params, train_data, 
                        categorical_feature=_categorical_columns_name,
                        num_boost_round=1000, 
                        valid_sets=(test_data,), 
                        valid_names=('valid',), 
                        verbose_eval=False, 
                        early_stopping_rounds=20)


    # DF, based on which importance is checked
    X_importance = X_val
    print("recuperando las variablwa impoetantes del modelo")
    # Explain model predictions using shap library:
    explainer = shap.TreeExplainer(lgb_model)
    print("TreeExplainer...done")
    shap_values = explainer.shap_values(X_importance)
    print("shap_values...done")
    
    if(objective=="multiclass"):
        shap_sum = np.abs(shap_values).mean(axis=0).mean(axis=0)
    else:  
        shap_sum = np.abs(shap_values).mean(axis=0).mean(axis=0)
    print("shap_sum...done")
    importance_df = pd.DataFrame([X_importance.columns.tolist(), shap_sum.tolist()]).T
    importance_df.columns = ['column_name', 'shap_importance']
    importance_df = importance_df.sort_values('shap_importance', ascending=False)
    

    
    zero_features = importance_df[importance_df['shap_importance']<=min_shap].column_name.values
    print("fin de : feature_sel_shap")
    return zero_features , importance_df



def hfm_predict_standalone(X_test,model,columns_to_ignore=[]):
    df_model = X_test.copy()

    #columns_to_ignore = ['ID_CLIENTE','ESTADO_DESEMBOLSO_COTIZA']
    #columns_to_ignore_plus = columns_to_ignore+zero_features.tolist()

    df_model.drop(columns_to_ignore,axis=1,inplace=True)


    le = preprocessing.LabelEncoder()
    for c in df_model.columns:
        col_type = df_model[c].dtype
        if col_type == 'object' or col_type.name == 'category':
            #print("column : "+str(c))
            df_model[c] = le.fit_transform(df_model[c])
            df_model[c] = df_model[c].astype('category') 


    #y_prob = model.predict_proba(df_model)[:, 1]
    y_pred = model.predict(df_model)
    return y_pred

def run_clasification(_X_test,_y_test,_model,_model_smote=None,doSMOTE=False,score_rs="f1",objective="binary"):


    colors = ['b','g','r','c','m','y','k','b']

    y_prob = _model.predict_proba(_X_test)[:, 1]
    #best_thr = op_sc.fmin(self.thr_to_accuracy, args=(_y_test, y_prob), x0=0.5)

    y_pred = _model.predict(_X_test)
    #y_pred = np.where(y_prob>=best_thr,1,0)


    if(doSMOTE):
        y_pred_smote = _model_smote.predict(_X_test)
        y_prob_smote = _model_smote.predict_proba(_X_test)[:, 1]

    mean_tpr = mean_smote_tpr = 0.0
    mean_fpr = mean_smote_fpr = np.linspace(0, 1, 100)


    print(classification_report(_y_test, y_pred))

    if(score_rs == 'roc_auc'):
        print('roc_auc_score: ',roc_auc_score(_y_test, y_pred))
    elif(score_rs == 'f1'):
         print('f1_score: ',f1_score(_y_test, y_pred))
    elif(score_rs == 'accuracy'):
         print('accuracy_score: ',accuracy_score(_y_test, y_pred))
    elif(score_rs == 'r2'):
         print('r2: ',r2_score(_y_test, y_pred))        
    elif(score_rs == 'neg_mean_squared_error'):
         print('mean_squared_error: ',mean_squared_error(_y_test, y_pred))   
    elif(score_rs == 'neg_mean_absolute_error'):
         print('neg_mean_absolute_error: ',neg_mean_absolute_error(_y_test, y_pred))      
    elif(score_rs == 'neg_mean_squared_log_error'):
         print('neg_mean_squared_log_error: ',neg_mean_squared_log_error(_y_test, y_pred))   
    elif(score_rs == 'neg_median_absolute_error'):
         print('neg_median_absolute_error: ',neg_median_absolute_error(_y_test, y_pred))  
    # print('recall_score: ',recall_score(_y_test, y_pred, average="binary"))
    # print('precision_score:',precision_score(_y_test, y_pred, average="binary"))
    # print('f1_score:',f1_score(_y_test, y_pred, average="binary"))

    if((score_rs=='roc_auc' or score_rs=='f1_score') and objective=="binary"):    
        # confusion matrix on the test data.
        print('\nMatriz de confucion para la data de prueba:')
        print(pd.DataFrame(confusion_matrix(_y_test, y_pred),
                        columns=['pred_neg', 'pred_pos'], index=['Negativo', 'Positivo']))


    test_indexes = _X_test.index.astype(int)
    probas = np.column_stack((test_indexes,y_prob*100,y_pred,_y_test))

    result = pd.DataFrame(data=probas[1:,1:],    # values
                            index=probas[1:,0],    # 1st column as index
                            columns=['RISK_SCORE','PREDICCION','REAL'])
    return result , _y_test , y_pred , y_prob




def hf_train_test_split( df=None,target_name=None,columns_to_drop=None, 
                        test_size=0.25,scaler=None,umbral_variables_similares=None,target_encoder=None):

    if(df is None or target_name is None):
        print("_df y target_name son parametros obligatorios")
    else:    
        df_model = df.copy()

        # Eliminando columnas no necesarias
        if(columns_to_drop is not None): 
            df_model.drop(columns_to_drop,axis=1,inplace=True)

        # Escalar datos numericos. Esto solo es valido para regresiones. Para modelos de arbol de desciciones no tiene efecto
        if(scaler is not None):    
            if(scaler=='StandardScaler'):
                std_scaler = StandardScaler()
            elif(scaler=='RobustScaler'):
                std_scaler = StandardScaler()
                #rob_scaler = RobustScaler()
            for idx,tp in enumerate(df_model.dtypes):
                if tp != 'object' and tp.name != 'category': 
                    df_model[df_model.columns[idx]] = std_scaler.fit_transform(df_model[df_model.columns[idx]].values.reshape(-1,1))


        # Convierte las variables categoricas en one-hot-encoding
            # columns_to_drop = []
            # for idx,tp in enumerate(df_model.dtypes):
            #     if tp == 'object' or tp.name == 'category': 
            #         df_dummies = pd.get_dummies(df_model[df_model.columns[idx] ],prefix=df_model.columns[idx] )
            #         df_model = pd.concat([df_model,df_dummies],axis=1)
            #         columns_to_drop.append(df_model.columns[idx])
            # df_model.drop(columns_to_drop,axis=1,inplace=True)  


        # Codificando y categorizando todas las variables de tipo    
        le = preprocessing.LabelEncoder()
        for c in df_model.columns:
            col_type = df_model[c].dtype
            if col_type == 'object' or col_type.name == 'category':
                #print("column : "+str(c))
                df_model[c] = le.fit_transform(df_model[c])
                df_model[c] = df_model[c].astype('category') 


        # aislando la variable dependiente de las independientes
        X = df_model.drop([target_name],1)   #Feature Matrix
        y = df_model.loc[:,target_name]
        #y_= df_model[[target_name]]
        if(target_encoder is not None and target_encoder):
            labels = preprocessing.LabelEncoder().fit_transform(y)
        else:
            labels = np.array(y)
        X_df = df_model.drop(target_name,1)
        students = df_model.index


        # Removiendo variables independientes que sean muy parecidas entre si
        if(umbral_variables_similares is not None):
            # Threshold for removing correlated variables
            threshold = umbral_variables_similares
            # Absolute value correlation matrix
            corr_matrix = X_df.corr().abs()
            # Upper triangle of correlations
            upper = corr_matrix.where(np.triu(np.ones(corr_matrix.shape), k=1).astype(np.bool))
            # Select columns with correlations above threshold
            to_drop = [column for column in upper.columns if any(upper[column] > threshold)]
            # Remove the columns
            X_df = X_df.drop(columns = to_drop)


        # separando la data en train, val y test
        X_train, X_test, y_train, y_test = train_test_split(X_df, labels, test_size=test_size, stratify=labels, random_state=1)

        X_train_, X_val, y_train_, y_val  = train_test_split(X_train, y_train, test_size=test_size, stratify=y_train, random_state=1)


        #X_train, X_test, y_train, y_test = train_test_split(X_df, labels,stratify=labels, test_size=0.25)
        #return train_test_split(X_df, labels,stratify=labels, test_size=_test_size)
        return X_train_, X_val , X_test , y_train_ , y_val , y_test


    



# -*- coding: utf-8 -*-

import core_helper.helper_general as hg
hg.set_base_path()

import lightgbm as lgb 
from scipy.stats import uniform as sp_uniform
import pandas as pd
from sklearn.model_selection import RandomizedSearchCV

#import general as g
import src.Prj_Core.core_helper.model.general as g

def get_pipeline():
    '''
    lgb_pipe = pipeline.Pipeline([
        #
        #  metric="None", first_metric_only=True -> ignora binary_logloss en early_stopping_rounds
        ('clf', lgb.LGBMClassifier(metric="None", first_metric_only=True, importance_type="gain",n_jobs=4))
        ])        

    '''
    lgb_pipe = lgb.LGBMClassifier(metric="None", 
                                  first_metric_only=True, 
                                  importance_type="gain",
                                  use_missing=True,
                                  zero_as_missing=False,
                                  n_jobs=4)
    return lgb_pipe




def get_possible_params():
     
    lgb_param_random = {
        'num_leaves':list(range(7, 50)),
        'feature_fraction':sp_uniform(loc=0.2, scale=0.8),
        'bagging_fraction':sp_uniform(loc=0.2, scale=0.8),   
        'learning_rate': [0.1,0.07,0.05,0.025,0.015,0.01,0.0085,0.007,0.005],
        'max_depth': [-1,1,3,5,10,15],              
    }

    return  lgb_param_random


def get_default_params():
    params = {}    
    params['metric']="None" 
    params['first_metric_only']=True 
    params['importance_type']="gain"
    params['use_missing']=True 
    params['zero_as_missing']=False 
    params['n_jobs']=4
    params['random_state']=1
    params['num_leaves']=15 
    params['feature_fraction']=0.7
    params['bagging_fraction']=0.7
    params['n_estimators']=10000 # optimizacion automatica con : early stopping
    params['objective']="binary" 
    params['bagging_freq']=1
    params['boosting_type']='gbdt'
    params['reg_alpha']=0
    params['reg_lambda']=0
        
    return params


def lgb_model(X_train,y_train,X_test,y_test,score_rs='average_precision',params=None):

    fn_eval = g.get_fn_eval(score_rs)
    fit_params = get_fit_params(X_train,y_train,X_test,y_test,fn_eval)
  
                       
    model = lgb.LGBMClassifier(**params)
    print('---------- definicion del modelo ----------------')
    print(model)
    print('-------------------------------------------------')
    
    model.fit(X_train,y_train,**fit_params)
   
    return model


def lgb_model_rscv(X_train, y_train, X_test, y_test,score_rs='average_precision', params=None, param_dist=None,n_iter=None,n_jobs = None):
    
    print("++++++++++++++++++++++++ INICIO - ESTIMAR MODELO ++++++++++++++++++++++++")   
    fn_eval = g.get_fn_eval(score_rs)
    fit_params = get_fit_params(X_train,y_train,X_test,y_test,fn_eval)          

    cv_ = g.get_kfold()          
    model = lgb.LGBMClassifier(**params)       
    print("------------------  n_jobs= ",n_jobs," -------------------")    
    print('------------------------ params -------------')
    print(params)
    print('------------------------------------------------')
    
    print('RandomizedSearchCV -> Multiples Hyperparametros')
    grid_obj = RandomizedSearchCV(estimator=model,
                                  param_distributions=param_dist,
                                  cv=cv_,
                                  n_iter=n_iter,
                                  refit=True,
                                  return_train_score=False,
                                  scoring = score_rs,
                                  n_jobs = n_jobs,
                                  verbose= True,          
                                  random_state=375)

    print('------------X_train-------------')
    print(X_train.shape)
    print(y_train.value_counts())
    print('---------------------------------')
    grid_obj.fit(X_train, y_train,**fit_params)

    estimator = grid_obj.best_estimator_
    results = pd.DataFrame(grid_obj.cv_results_)
    best_params = grid_obj.best_params_  
    
    print('------------mejores hyperparametros-------------')
    print(best_params)
    print('------------------------------------------------')
    print("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++") 
    return results, estimator , best_params

def get_fit_params(X_train,y_train,X_test, y_test,fn_eval):
 
    fit_params={"early_stopping_rounds":800, 
                "eval_metric" : fn_eval, 
                "eval_set" : [(X_train,y_train),(X_test,y_test)],
                'eval_names': ['train','test'],
                'verbose': 200,
                'feature_name': 'auto', # that's actually the default
                'categorical_feature': 'auto' # that's actually the default
               }    
    return fit_params


# -*- coding: utf-8 -*-
import core_helper.helper_general as hg
hg.set_base_path()

#import general as g
import src.Prj_Core.core_helper.model.general as g

#import lgb_model as l
import src.Prj_Core.core_helper.model.lgb_model as l

import math
from imblearn.under_sampling import RandomUnderSampler
import numpy as np
import pandas as pd
import core_helper.helper_plot as  hp
import src.Prj_Core.core_helper.model.general as g
import time

def modelar(X_train,y_train,X_test,y_test,url):
    start = time.time()
    list_m = custom_bagging_model(X_train,y_train,X_test,y_test,K=1) 
    g.save_model(list_m,url)
    y_pred,y_prob_uno , predicted_probas = predict_proba(list_m,X_test)
        
    #kpis = hp.print_kpis_rendimiento_modelo(y_test,predicted_probas,url)  
    #g.get_summary_evaluation2(X_test,predicted_probas,y_test,url)
    
    #y_pred_t,y_prob_uno_t , predicted_probas_t = cbm_predict(list_m,X_t)
    #y_pred_t_mas_1,y_prob_uno_t_mas_1 , predicted_probas_t_mas_1 = cbm_predict(list_m,X_t_mas_1)
    
    return list_m , predicted_probas

'''
def custom_bagging_pipeline_model(X_train,y_train,X_test,y_test,X_t,X_t_mas_1,url):
    start = time.time()
    list_m = custom_bagging_model(X_train,y_train,X_test,y_test,K=1) 
    y_pred,y_prob_uno , predicted_probas = predict_proba(list_m,X_test)
    
    
    kpis = hp.print_kpis_rendimiento_modelo(y_test,predicted_probas,url)  
    g.get_summary_evaluation2(X_test,predicted_probas,y_test,url)
    
    y_pred_t,y_prob_uno_t , predicted_probas_t = predict_proba(list_m,X_t)
    y_pred_t_mas_1,y_prob_uno_t_mas_1 , predicted_probas_t_mas_1 = predict_proba(list_m,X_t_mas_1)
    
    return kpis , predicted_probas , y_prob_uno_t, y_prob_uno_t_mas_1
'''


def predict_proba(list_model,X_test):
    list_y_prob = []
    for model in list_model:
        #y_pred = model.predict(X_test)
        predicted_probas = model.predict_proba(X_test)        
        list_y_prob.append(predicted_probas[:,1])
    y_prob_uno  = np.mean(list_y_prob, axis=0)
    y_prob_cero = [1-x for x in y_prob_uno]
    predicted_probas = np.vstack((y_prob_cero, y_prob_uno)).T
    y_pred = np.round(y_prob_uno, 0)
    return y_pred,y_prob_uno , predicted_probas
 


def custom_bagging_model(X_train,y_train,X_test,y_test,score_rs='average_precision',params=None,alpha=None, 
                         n_iter_rscv=None,K = 1):
    
    
    #MIN_N_TRAIN = 10000 #numero minimo de registros para que lgb no sobre ajuste

    N_p = y_train[y_train==1].count()
    N_n = y_train[y_train==0].count()
    
    MIN_N_TRAIN = get_Total_min_to_train(N_p)
    
    fn_eval = g.get_fn_eval(score_rs)
    fit_params = l.get_fit_params(X_train,y_train,X_test,y_test,fn_eval)
        
    alpha=(MIN_N_TRAIN-N_p)/(N_n) 
    N_n_res = int(round(alpha*N_n,0))
    
    list_model = []

    lambda_ = N_p/N_n

    T = int(round(math.log(0.05)/math.log(1-K*alpha)))
    T,K = get_T_reducido(K,alpha)
    print("T: ",T,"k:",K)

    for i in range(T):
        print("model :",i)

        rus = RandomUnderSampler(random_state=i,sampling_strategy={1:N_p ,0: N_n_res},replacement=False) #majority
        X_res, y_res = rus.fit_resample(X_train, y_train)
        print("total: ",len(y_res),", Class_1: ",y_res[y_res==1].count(),", Class_0: ",y_res[y_res==0].count())
       

        if n_iter_rscv is None:
            if params is None:
                params = l.get_default_params()   
                
            params['pos_bagging_fraction']=1
            params['neg_bagging_fraction']=1
            params['scale_pos_weight']=(K*alpha*N_n)/N_p
            
            model = l.lgb_model(X_res,y_res,X_test,y_test,score_rs=score_rs,params=params)  
        else:
            results, model = l.lgb_model_rscv(X_res, y_res, X_test, y_test,score_rs=score_rs, n_iter=n_iter_rscv)   

        list_model.append(model)
  
    return list_model



def get_Total_min_to_train(N_p):
    T_minimo = 10000 
    
    while N_p>=T_minimo:
      T_minimo = T_minimo + 30000

    return T_minimo



def get_T_reducido(K,alpha):
    T = get_T(K,alpha)   
    
    if(T>20):
        K = K +1
        T = get_T(K,alpha)
        
        if(T>20):
            K = K +1
            T = get_T(K,alpha)
            
            if(T>20):
                K = K +1
                T = get_T(K,alpha)
        
        return T, K 
    else:
        return T, K
    
    

def get_T(K,alpha):
    T = int(round(math.log(0.05)/math.log(1-K*alpha)))
    return T



import time
import json
import numpy as np
import pandas as pd
import plotly.graph_objects as go
import urllib.request
from simpledbf import Dbf5
import itertools
import seaborn as sns
import os
from sklearn.model_selection import train_test_split
import pandas as pd
from sklearn import pipeline
import lightgbm as lgb ## cambias

from sklearn.preprocessing import StandardScaler, LabelEncoder , LabelBinarizer
from simpledbf import Dbf5
from scipy.stats import randint as sp_randint
from scipy.stats import uniform as sp_uniform
from helper_transformers import CatTransformer


def get_pipeline():

    lgb_pipe = pipeline.Pipeline([
        #
        #  metric="None", first_metric_only=True -> ignora binary_logloss en early_stopping_rounds
        ('clf', lgb.LGBMClassifier(metric="None", first_metric_only=True, importance_type="gain",n_jobs=4))
        ])


    lgb_param_random = {
        'clf__use_missing':[True],
        'clf__zero_as_missing':[False],
        
        'clf__learning_rate': [0.015,0.012,0.025,0.01,0.0085,0.007,0.005],
        'clf__objective':['binary'],
        #'clf__max_depth': [2,3,5,12,22,30,50,63],
        'clf__max_depth': [-1], 
        
        #'clf__n_estimators': [150,180,200,250,275,300,350,400,450,500,550,600,650,700,750,800,850,900],
        'clf__n_estimators': [70000],
        
        'clf__num_leaves': list(range(7, 50)),  
        'clf__min_data_in_leaf': sp_randint(100, 500),  
        #'clf__min_sum_hessian_in_leaf': [1e-5, 1e-3, 1e-2, 1e-1, 1, 1e1, 1e2, 1e3, 1e4],
        #'clf__min_data_in_leaf': [50,60,70,80,90,100,110], 
        #'clf__max_bin': [25],        
                              
        'clf__boosting_type': ['gbdt'],
        #'clf__reg_lambda': [0.01, 0.1, 1],
        'clf__reg_alpha': [0, 1e-1, 1, 2, 5, 7, 10, 50, 100],
        'clf__reg_lambda': [0, 1e-1, 1, 5, 10, 20, 50, 100],
        
        #'clf__scale_pos_weight':scale, 
        'clf__feature_fraction':sp_uniform(loc=0.4, scale=0.6),
        'clf__bagging_fraction':sp_uniform(loc=0.2, scale=0.8), 
        #'clf__bagging_fraction':[0.3,0.4,0.5,0.75,0.85,0.9,0.95],
        #'clf__bagging_freq':[2,5,10,15,20,25]# debe ser superior a 0 y menos a 1 para activar bagging_fraction       
        'clf__bagging_freq':[1]# debe ser superior a 0 y menos a 1 para activar bagging_fraction       
        
    }

    return lgb_pipe, lgb_param_random

def tranform_data(df_join,df_join_eval,desercion_eval):
    
    df_join['EDAD_EN_DECIMALES_T']  = df_join['EDAD_EN_DECIMALES_T'].round()
    df_join_eval['EDAD_EN_DECIMALES_T']  = df_join_eval['EDAD_EN_DECIMALES_T'].round()
    
    '''
    df_join['Z_NOTA_MAT_T_MENOS_1']  = df_join['Z_NOTA_MAT_T_MENOS_1'].round()
    df_join['Z_NOTA_OTROS_T_MENOS_1']  = df_join['Z_NOTA_OTROS_T_MENOS_1'].round()
    df_join['Z_NOTA_COM_T_MENOS_1']  = df_join['Z_NOTA_COM_T_MENOS_1'].round()
    df_join['COST_APAFA_T_MENOS_1']  = df_join['COST_APAFA_T_MENOS_1'].round()
    df_join['STD_TRASLADOS']  = df_join['STD_TRASLADOS'].round()       
    
    df_join_eval['Z_NOTA_MAT_T_MENOS_1']  = df_join_eval['Z_NOTA_MAT_T_MENOS_1'].round()
    df_join_eval['Z_NOTA_OTROS_T_MENOS_1']  = df_join_eval['Z_NOTA_OTROS_T_MENOS_1'].round()
    df_join_eval['Z_NOTA_COM_T_MENOS_1']  = df_join_eval['Z_NOTA_COM_T_MENOS_1'].round()
    df_join_eval['COST_APAFA_T_MENOS_1']  = df_join_eval['COST_APAFA_T_MENOS_1'].round()
    df_join_eval['STD_TRASLADOS']  = df_join_eval['STD_TRASLADOS'].round()
    '''
               
            
    column_y = 'DESERCION'
    columns_with_nan = ["JUSTIFICACION_RETIRO_T_MENOS_1","DSC_PAIS","DSC_LENGUA"]#'LENGUA_APOD','ID_PERSONA',
    columns_to_drop = ['COD_MOD_T','ANEXO_T','COD_MOD_T_MENOS_1','ANEXO_T_MENOS_1','ID_GRADO_T_MENOS_1',
                       'NUMERO_DOCUMENTO_APOD','N_DOC','NIVEL_INSTRUCCION_APOD',
                       #'ID_NIVEL','JUSTIFICACION_RETIRO_N_MENOS_1', 
                       #,'NIVEL_INSTRUCCION_APOD_SHORT','ANIOS_ESCOLARIDAD_APOD',
                       #'ID_GRADO_N_MENOS_1','TIPO_DOC','N_DOC',
                       'NOTA_MATEMATICA_X_ALUMNO','NOTA_COMUNICACION_X_ALUMNO','MEAN_NOTA_MATEMATICA_X_CODMOD_NVL_GR',
                       'NOTA_OTROS_X_ALUMNO',
                       'MEAN_NOTA_COMUNICACION_X_CODMOD_NVL_GR','STD_NOTA_MATEMATICA_X_CODMOD_NVL_GR',
                       'STD_NOTA_COMUNICACION_X_CODMOD_NVL_GR',
                       'MEAN_NOTA_OTROS_X_CODMOD_NVL_GR','STD_NOTA_OTROS_X_CODMOD_NVL_GR',#'NOTA_OTROS_X_ALUMNO',
                      ] #,
    cat_muy_largas = ['D_REGION']

    columns_to_drop_all = columns_with_nan+columns_to_drop +cat_muy_largas+[column_y]

    y = df_join[column_y] #DESERCION_2016_2018, DESERCION_2016_2017_2018 DESERCION_2016_2017

    y_eval = None
    if desercion_eval:
        y_eval = df_join_eval[column_y]
        columns_to_drop_all_eval = columns_to_drop_all
    else:
        columns_to_drop_all_eval = columns_with_nan+columns_to_drop +cat_muy_largas




    X = df_join.drop(columns = columns_to_drop_all) 
    ID_PERSONA_EVAL = df_join_eval.ID_PERSONA
    X_eval = df_join_eval.drop(columns = columns_to_drop_all_eval) 


    #X['PALOMITA'] = np.random.randint(1,100, X.shape[0])
    #X_eval['PALOMITA'] = np.random.randint(1,100, X_eval.shape[0])

    ct = CatTransformer(preprocessing="lb")#lb le
    ct.fit(X)
    X_t = ct.transform(X)

    ct = CatTransformer(preprocessing="lb")#lb le
    ct.fit(X_eval)
    X_t_eval = ct.transform(X_eval)

    X_t.drop(['SISFOH_CSE_OTROS'], axis = 1,inplace=True) 
    X_t_eval.drop(['SISFOH_CSE_OTROS'], axis = 1,inplace=True) 
      
    X_t = reduce_mem_usage(X_t)    
    X_train, X_test, y_train, y_test = train_test_split(X_t, y, test_size=0.20,stratify=y,random_state=42)
    
    #X_train['PALOMITA'] = np.random.randn(X_train.shape[0])
    #X_test['PALOMITA'] = np.random.randn(X_test.shape[0])
    
    return X_train, X_test, y_train, y_test , X_t_eval, y_eval,ID_PERSONA_EVAL








def get_summary_evaluation(X,estimator_random,y,url_dir):

    X_eval = X.copy()
    X_eval['PREDICT_PROBA'] = estimator_random.predict_proba(X)[:,1]
    X_eval['PREDICT'] = estimator_random.predict(X)
    if y is not None:
        X_eval['REAL'] =  y

    criteria = [X_eval['PREDICT_PROBA'].between(0,0.1),
                X_eval['PREDICT_PROBA'].between(0.1, 0.2), 
                X_eval['PREDICT_PROBA'].between(0.2, 0.3),
                X_eval['PREDICT_PROBA'].between(0.3, 0.4),
                X_eval['PREDICT_PROBA'].between(0.4, 0.5),
                X_eval['PREDICT_PROBA'].between(0.5, 0.6),
                X_eval['PREDICT_PROBA'].between(0.6, 0.7),
                X_eval['PREDICT_PROBA'].between(0.7, 0.8),
                X_eval['PREDICT_PROBA'].between(0.8, 0.9),
                X_eval['PREDICT_PROBA'].between(0.9,0.95),
                X_eval['PREDICT_PROBA'].between(0.95, 1)
               ]

    values = ['0% - 10%', '10% - 20%', '20% - 30%','30% - 40%',
              '40% - 50%','50% - 60%','60% - 70%','70% - 80%','80% - 90%','90% - 95%','95% - 100%']

    values_index = [1, 2,3,4,5,6,7,8,9,10,11]
    
    ###################################################################
    
    criteria2 = [X_eval['PREDICT_PROBA'].between(0.0, 0.5),
                X_eval['PREDICT_PROBA'].between(0.5, 1),             
               ]

    values2 = ['0% - 50%', '50% - 100%']

    values_index2 = [1, 2]
    
    #####################################################
 
    X_eval['PREDICT_CATEGORY'] = np.select(criteria, values, 0)
    X_eval['PREDICT_CATEGORY_INDEX'] = np.select(criteria, values_index, 0)

    X_eval['PREDICT_CATEGORY2'] = np.select(criteria2, values2, 0)
    X_eval['PREDICT_CATEGORY_INDEX2'] = np.select(criteria2, values_index2, 0)

    X_eval['TOTAL'] = 1 
    
    if y is not None:
    
        X_eval['ACIERTO(VP)'] = np.where( (X_eval['REAL']==1) & (X_eval['PREDICT']==1), 1 ,0)
        X_eval['FP'] = np.where( (X_eval['REAL']==0) & (X_eval['PREDICT']==1), 1 ,0)    

        X_eval_gb = X_eval.groupby(['PREDICT_CATEGORY_INDEX','PREDICT_CATEGORY']).agg({'TOTAL':'count','REAL':'sum'}).reset_index()
        X_eval_gb.sort_values(by='PREDICT_CATEGORY_INDEX', ascending=False,inplace=True)
        X_eval_gb.rename(columns={'REAL': 'DESERTORES','PREDICT_CATEGORY':'RANGO'}, inplace=True)
        del X_eval_gb['PREDICT_CATEGORY_INDEX']
        X_eval_gb.reset_index(drop=True,inplace=True)

        X_eval_gb['TOTAL %']=round(X_eval_gb['TOTAL']/X_eval_gb['TOTAL'].sum(),3)
        X_eval_gb['COBERTURA %']=round(X_eval_gb['DESERTORES']/X_eval_gb['DESERTORES'].sum(),3)
        
        
        X_eval_gb['PRECISION RANGO %']=round(X_eval_gb['DESERTORES']/X_eval_gb['TOTAL'],3)

        #X_eval_gb.index.name = 'N'

        
        
        if not url_dir:
            url_dir="reporte_modelo/out/"
        else:

            url_dir = '{}/{}'.format("reporte_modelo/out",url_dir)
            if not os.path.exists(url_dir):
                os.makedirs(url_dir)

        X_eval_gb.to_excel(url_dir+"df_agg_t.xlsx", encoding="utf-8")

        
        
        
        # Set colormap equal to seaborns light green color palette
        cm = sns.light_palette("red", as_cmap=True)

        return X_eval, (X_eval_gb.style
          .background_gradient(cmap=cm, subset=['PRECISION RANGO %'])
          .format({'TOTAL %': "{:.1%}"})  
          .format({'PRECISION RANGO %': "{:.1%}"})
          .format({'COBERTURA %': "{:.1%}"})
        )
    
    else:
        X_eval_gb = X_eval.groupby(['PREDICT_CATEGORY_INDEX','PREDICT_CATEGORY']).agg({'TOTAL':'count'}).reset_index()
        X_eval_gb.sort_values(by='PREDICT_CATEGORY_INDEX', ascending=False,inplace=True)
        X_eval_gb.rename(columns={'PREDICT_CATEGORY':'RANGO'}, inplace=True)
        del X_eval_gb['PREDICT_CATEGORY_INDEX']
        
        X_eval_gb['TOTAL %']=round(X_eval_gb['TOTAL']/X_eval_gb['TOTAL'].sum(),3)
        
        X_eval_gb.reset_index(drop=True,inplace=True)
        
        
        cm = sns.light_palette("blue", as_cmap=True)
        return X_eval, (X_eval_gb.style
          .background_gradient(cmap=cm, subset=['TOTAL %'])
          .format({'TOTAL %': "{:.1%}"})      
        )
        
#df_nom,df_agg = get_summary_evaluation(X_test,estimator_random,y_test)
#df_agg

#desertores predichos / desertores reales






def save_join_data(df,df_eval,url_dir=""):
    
    if not url_dir:
        url_dir="reporte_modelo/out/"
    else:
        url_dir = '{}/{}'.format("reporte_modelo/out",url_dir)
        if not os.path.exists(url_dir):
            os.makedirs(url_dir)
        print("reporte generado en : "+url_dir)
    
    specific_url = url_dir+"data.csv"
    specific_url_eval = url_dir+"data_eval.csv"
    
    df.to_csv(specific_url,index =False , encoding="utf-8") 
    df_eval.to_csv(specific_url_eval,index =False , encoding="utf-8") 
    
    



    

def exportar_para_dashborad(df_nom,df_nominal_eval,df_servicios,ID_PERSONA_EVAL,url_dir=""):

    df_result =pd.DataFrame({'ID_PERSONA':ID_PERSONA_EVAL.values,'RISK_SCORE':df_nom.PREDICT_PROBA.values}) 
    df_result.shape
    
    mapa_des_nivel = {'A1':'Inicial - Cuna','A2':'Inicial - Jardín',
                      'A3':'Inicial - Cuna-jardín',
                      'A5':'Inicial - Programa no escolarizado',
                      'B0':'Primaria','F0':'Secundaria'}
    
    mapa_gestion =   {1:'PÚBLICA',2:'PÚBLICA',3:'PRIVADA'}
    
    
    df_result_nom = pd.merge(df_result , df_nominal_eval, left_on=['ID_PERSONA'], right_on=['ID_PERSONA'], how='inner')
    df_result_nom['DSC_NIVEL'] = df_result_nom.ID_NIVEL.map(mapa_des_nivel)



    df_result_nom_geo = pd.merge(df_result_nom , df_servicios, left_on=['COD_MOD','ANEXO'],
                                 right_on=['COD_MOD','ANEXO'], how='inner')
    
    df_result_nom_geo['GESTION_DES'] = df_result_nom_geo.GESTION.map(mapa_gestion)


    df_result_nom_geo.rename(columns={'EDAD_AL_2020': 'EDAD','N_DOC':'DNI','CODGEO':'COD_GEO',
                                      'CODIGO_ESTUDIANTE':'COD_ESTUDIANTE' }, inplace=True)
    
    cls_export = ["COD_MOD","ANEXO","COD_GEO","GESTION_DES","ID_NIVEL","DSC_NIVEL","ID_GRADO","DSC_GRADO",
                  "ID_PERSONA","COD_ESTUDIANTE",
                  "DNI","SEXO","EDAD","APELLIDO_PATERNO","APELLIDO_MATERNO","NOMBRES","RISK_SCORE"]
    
    #specific_url = "reporte_modelo/out/"+url_dir+"listado_eval.xlsx"
    
    
    if not url_dir:
        url_dir="reporte_modelo/out/"
    else:

        url_dir = '{}/{}'.format("reporte_modelo/out",url_dir)
        if not os.path.exists(url_dir):
            os.makedirs(url_dir)
        print("reporte generado en : "+url_dir)
    
    specific_url = url_dir+"listado_eval.csv"
    
    #df_result_nom_geo[cls_export].to_excel(specific_url,index=False) 
    df_result_nom_geo[cls_export].to_csv(specific_url,index =False , encoding="utf-8")
    

    

   
    
def get_url_homo_nom(ANIO,ID_GRADO):
    
    if ANIO>=2019:
        if ID_GRADO>=10:
            url_nom = "../PROYECTO_AVISE/00.CARGA/00.HOMOGENEO_NOMINAL_SIAGIE/NOMINAL_F0_{}.csv"
        if ID_GRADO>=4 and ID_GRADO<=9:
            url_nom = "../PROYECTO_AVISE/00.CARGA/00.HOMOGENEO_NOMINAL_SIAGIE/NOMINAL_B0_{}.csv"
    else:
        url_nom = "../PROYECTO_AVISE/00.CARGA/00.HOMOGENEO_NOMINAL_SIAGIE/NOMINAL_{}.csv"
    
    return url_nom.format(ANIO)


def load_homogeneo_nominal(ANIO,ID_GRADO):
    
    dtypes_columns = {'COD_MOD': str,
                  'ANEXO':int,
                  'UBIGEO_NACIMIENTO_RENIEC':str,
                  'N_DOC':str,
                  'CODIGO_ESTUDIANTE':str,
                  'NUMERO_DOCUMENTO':str,
                  'NUMERO_DOCUMENTO_APOD':str,
                  'CODOOII':str
                  }
    
    columns_n         = ['COD_MOD', 'ANEXO', 'ID_NIVEL','ID_GRADO','DSC_GRADO','ID_PERSONA','CODIGO_ESTUDIANTE','N_DOC',
                         'SEXO','EDAD_AL_2019','APELLIDO_PATERNO','APELLIDO_MATERNO','NOMBRES'
                         ]  
    
    
    iter_csv_n = pd.read_csv(get_url_homo_nom(ANIO,ID_GRADO),encoding="latin-1",sep="|",
                               dtype=dtypes_columns,iterator=True, chunksize=100000,usecols=columns_n)
    
    dataSet_n = pd.concat([chunk[chunk['ID_GRADO'] == ID_GRADO] for chunk in iter_csv_n])
    if ANIO==2020:
        dataSet_n.rename(columns={'EDAD_AL_2019': 'EDAD_AL_2020' }, inplace=True)
    return dataSet_n


def get_df_standarized(X_test,X_t_eval):
    columns_to_add = list(set(X_test.columns.tolist()) - set(X_t_eval.columns.tolist()))
    columns_to_delete = list(set(X_t_eval.columns.tolist()) - set(X_test.columns.tolist()))

    for column in columns_to_add:        
        X_t_eval[column]=0    

    for column in columns_to_delete:
        if column in X_t_eval.columns:
            del X_t_eval[column]

    return X_t_eval[X_test.columns.tolist()]  


import pandas as pd
import numpy as np
from functools import reduce
from simpledbf import Dbf5
import gc
import time
import os
from helper_functions import reduce_mem_usage

def get_variables_altamente_correlacionadas(X_train):
    threshold = 0.9
    # Absolute value correlation matrix
    corr_matrix = X_train.corr().abs()
    upper = corr_matrix.where(np.triu(np.ones(corr_matrix.shape), k=1).astype(np.bool))
    to_drop = [column for column in upper.columns if any(upper[column] > threshold)]
    return to_drop


def get_ids_por_anio(anio):       
    dtypes_columns = {
                      'ID_PERSONA':int,#nurvo                    
                      }           
    columns_n_ = ['ID_PERSONA'] 
    if(anio<2019):
        url = "../PROYECTO_AVISE/00.CARGA/00.HOMOGENEO_NOMINAL_SIAGIE/NOMINAL_{}.csv"
        df = pd.read_csv(url.format(anio), usecols=columns_n_,encoding="latin-1",sep="|",dtype=dtypes_columns)
    else:
        list_df = []
        for nivel in ["A0","B0","F0"]:
            url = "../PROYECTO_AVISE/00.CARGA/00.HOMOGENEO_NOMINAL_SIAGIE/NOMINAL_{}_{}.csv"
            ds_2020 = pd.read_csv(url.format(nivel,anio), usecols=columns_n_,encoding="latin-1",
                                  sep="|",dtype=dtypes_columns)
            list_df.append(ds_2020)
        df = pd.concat(list_df)
    return df

def get_participacion_juntos_por_anio(anio):       
    dtypes_columns = {
                      'ID_PERSONA':int,#nurvo
                      'JUNTOS':int,
                      }           
    columns_n_ = ['ID_PERSONA','JUNTOS'] 
    if(anio<2019):
        url = "../PROYECTO_AVISE/00.CARGA/00.HOMOGENEO_NOMINAL_SIAGIE/NOMINAL_{}.csv"
        df = pd.read_csv(url.format(anio), usecols=columns_n_,encoding="latin-1",sep="|",dtype=dtypes_columns)
    else:
        list_df = []
        for nivel in ["A0","B0","F0"]:
            url = "../PROYECTO_AVISE/00.CARGA/00.HOMOGENEO_NOMINAL_SIAGIE/NOMINAL_{}_{}.csv"
            ds_2020 = pd.read_csv(url.format(nivel,anio), usecols=columns_n_,encoding="latin-1",
                                  sep="|",dtype=dtypes_columns)
            list_df.append(ds_2020)
        df = pd.concat(list_df)
    return df


def get_situacion_final_recuperacion_por_anio(anio):       
    dtypes_columns = {
                      'ID_PERSONA':int,#nurvo
                      'SITUACION_FINAL':str,
                      'SF_RECUPERACION':str,
                      }           
    columns_n_ = ['ID_PERSONA','SITUACION_FINAL','SF_RECUPERACION'] 
    if(anio<2019):
        url = "../PROYECTO_AVISE/00.CARGA/00.HOMOGENEO_NOMINAL_SIAGIE/NOMINAL_{}.csv"
        df = pd.read_csv(url.format(anio), usecols=columns_n_,encoding="latin-1",sep="|",dtype=dtypes_columns)
    else:
        list_df = []
        for nivel in ["A0","B0","F0"]:
            url = "../PROYECTO_AVISE/00.CARGA/00.HOMOGENEO_NOMINAL_SIAGIE/NOMINAL_{}_{}.csv"
            ds_2020 = pd.read_csv(url.format(nivel,anio), usecols=columns_n_,encoding="latin-1",
                                  sep="|",dtype=dtypes_columns)
            list_df.append(ds_2020)
        df = pd.concat(list_df)
    return df

def generar_kpis_sfr(df,ANIO,T_ANIOS):
    
    ID_PERSONA_SERIES = df['ID_PERSONA']
    ultimo_anio =ANIO- T_ANIOS

    num = 1
    list_df_m_tras = []
    cols_to_sum_des = []
    cols_to_sum_ret = []
    cols_to_drop = []
    ANIO = ANIO-1
    for anio in range(ANIO,ultimo_anio,-1):
        if(anio<=2013):
            break
        col_name_des="DESAPROBADO_T"
        col_name_ret="RETIRADO_T"
        
        if(num>0):
            posfix="_MENOS_{}".format(num)
            col_name_des = col_name_des+posfix
            col_name_ret = col_name_ret+posfix
            
        cols_to_drop.append(col_name_des)
        cols_to_drop.append(col_name_ret)
        
        cols_to_sum_des.append(col_name_des)
        cols_to_sum_ret.append(col_name_ret)
        
        df_m_t = pd.merge(ID_PERSONA_SERIES, get_situacion_final_recuperacion_por_anio(anio),left_on="ID_PERSONA",
                          right_on="ID_PERSONA", how='left')
        
        df_m_t[col_name_des] = np.where((df_m_t['SITUACION_FINAL']=='DESAPROBADO') |
                                                (df_m_t['SF_RECUPERACION']=='DESAPROBADO'),1,0)
        
        df_m_t[col_name_ret] = np.where((df_m_t['SITUACION_FINAL']=='RETIRADO'),1,0)
        
        df_m_t.drop(['SITUACION_FINAL','SF_RECUPERACION'], axis = 1,inplace=True)  

        list_df_m_tras.append(df_m_t)
        num+=1

    df_final = reduce(lambda left,right: pd.merge(left,right,on='ID_PERSONA'), list_df_m_tras)
    df_final['TOTAL_DESAPROBADO'] = df_final[cols_to_sum_des].sum(axis=1)
    df_final['MEAN_DESAPROBADO'] = df_final[cols_to_sum_des].mean(axis=1)
    df_final['STD_DESAPROBADO'] = df_final[cols_to_sum_des].std(axis=1)
    
    
    df_final['TOTAL_RETIRADO'] = df_final[cols_to_sum_ret].sum(axis=1)
    df_final['MEAN_RETIRADO'] = df_final[cols_to_sum_ret].mean(axis=1)
    df_final['STD_RETIRADO'] = df_final[cols_to_sum_ret].std(axis=1)
    
    df_final.drop(cols_to_drop, axis = 1,inplace=True)    
    
    #df = pd.merge(df, df_final, left_on=["ID_PERSONA"],  right_on=["ID_PERSONA"],  how='inner')
    
    return df_final


def generar_kpis_juntos(df,ANIO,T_ANIOS):
    
    ID_PERSONA_SERIES = df['ID_PERSONA']
    ultimo_anio =ANIO- T_ANIOS

    num = 0
    list_df_m_tras = []
    cols_to_sum = []
    cols_to_drop = []
    for anio in range(ANIO,ultimo_anio,-1):
        if(anio<=2013):
            break
        col_name="JUNTOS_T"
        if(num>0):
            posfix="_MENOS_{}".format(num)
            col_name = col_name+posfix
        cols_to_drop.append(col_name)
        cols_to_sum.append(col_name)
        df_m_t = pd.merge(ID_PERSONA_SERIES, get_participacion_juntos_por_anio(anio),left_on="ID_PERSONA",
                          right_on="ID_PERSONA", how='left')
        df_m_t.fillna({'JUNTOS':0}, inplace=True)
        df_m_t.rename(columns={'JUNTOS': col_name}, inplace=True)
        list_df_m_tras.append(df_m_t)
        num+=1

    df_final = reduce(lambda left,right: pd.merge(left,right,on='ID_PERSONA'), list_df_m_tras)
    df_final['TOTAL_JUNTOS'] = df_final[cols_to_sum].sum(axis=1)
    df_final['MEAN_JUNTOS'] = df_final[cols_to_sum].mean(axis=1)
    df_final['STD_JUNTOS'] = df_final[cols_to_sum].std(axis=1)
    df_final.drop(cols_to_drop, axis = 1,inplace=True)    
    
    #df = pd.merge(df, df_final, left_on=["ID_PERSONA"],  right_on=["ID_PERSONA"],  how='inner')
    
    return df_final

def add_kpis_general(df,df_kpis,how='inner'):
    df = pd.merge(df, df_kpis, left_on=["ID_PERSONA"],  right_on=["ID_PERSONA"],  how=how)
    return df

def get_desertores_por_anio(anio):
    url = "../PROYECTO_AVISE/01.DESERCION/DESERCION_{}"
    anios_str=str(anio)+"_"+str(anio+1)
    ds_2020 = pd.read_csv(url.format("{}.csv".format(anios_str)))
    #print(anios_str)
    return ds_2020

def generar_kpis_desercion(df,ANIO,T_ANIOS):
    
    ID_PERSONA_SERIES = df['ID_PERSONA']
    ultimo_anio =ANIO- T_ANIOS

    num = 1
    list_df_m_tras = []
    cols_to_sum = []
    cols_to_drop = []
    ANIO = ANIO-1
    for anio in range(ANIO,ultimo_anio,-1):
        if(anio<=2013):
            break
        col_name="DESERTARA_DESPUES_DE_T"
        if(num>0):
            posfix="_MENOS_{}".format(num)
            col_name = col_name+posfix
            cols_to_drop.append(col_name)
        cols_to_sum.append(col_name)
        df_m_t = pd.merge(ID_PERSONA_SERIES, get_desertores_por_anio(anio),left_on="ID_PERSONA",
                          right_on="ID_PERSONA", how='left')
        anios_str=str(anio)+"_"+str(anio+1)
        original_col = "DESERCION_"+anios_str

        df_m_t.fillna({original_col:0}, inplace=True)
        df_m_t.rename(columns={original_col: col_name}, inplace=True)

        list_df_m_tras.append(df_m_t)
        num+=1

    df_final = reduce(lambda left,right: pd.merge(left,right,on='ID_PERSONA'), list_df_m_tras)
    df_final['TOTAL_DESERCIONES'] = df_final[cols_to_sum].sum(axis=1)
    df_final['MEAN_DESERCIONES'] = df_final[cols_to_sum].mean(axis=1)
    df_final['STD_DESERCIONES'] = df_final[cols_to_sum].std(axis=1)
    df_final.drop(cols_to_drop, axis = 1,inplace=True)    
    
    #df = pd.merge(df, df_final, left_on=["ID_PERSONA"],  right_on=["ID_PERSONA"],  how='inner')
    
    return df_final

def get_traslados_por_anio(anio,TIPO_TRASLADO='EN EL MISMO AÑO'):
    url_trasl = '../PROYECTO_AVISE/DATA/DATA_SIAGIE/Siagie_Traslados_{}.txt'.format(anio)
    sep = "|"
    encoding = 'latin-1'
    cols_tras = ['ID_PERSONA','TIPO_TRASLADO']

    #df_trasl = pd.read_csv(url_trasl ,encoding='utf-8',usecols=cols_tras,  sep=sep,dtype={'PERSONA_NRO_DOC':str})
    df_trasl = pd.read_csv(url_trasl ,encoding=encoding,usecols=cols_tras,  sep=sep,dtype={'ID_PERSONA':int})
    #if(anio==2019):
        #df_trasl = df_trasl[df_trasl.TIPO_TRASLADO==TIPO_TRASLADO].copy()
        #df_trasl.reset_index(drop=True,inplace=True)
 
    df_agg_t  = df_trasl.assign(
     TOTAL_TRASLADOS =   1
    ).groupby(['ID_PERSONA']).agg({'TOTAL_TRASLADOS':'sum'})

    df_agg_t.sort_values(by='TOTAL_TRASLADOS', ascending=False,inplace=True)
    df_agg_t.reset_index(inplace=True)
    
    return df_agg_t

def get_traslados_a_publico(anio,df_servicios):
    url_trasl = '../PROYECTO_AVISE/DATA/DATA_SIAGIE/Siagie_Traslados_{}.txt'.format(anio)
    sep = "|"
    encoding = 'latin-1'
    cols_tras = ['ID_PERSONA','TIPO_TRASLADO','COD_MOD_ORIGEN','ANEXO_ORIGEN','COD_MOD_DESTINO','ANEXO_DESTINO']

    cl_s = ["COD_MOD","ANEXO","ES_PUBLICO"]
    #df_trasl = pd.read_csv(url_trasl ,encoding='utf-8',usecols=cols_tras,  sep=sep,dtype={'PERSONA_NRO_DOC':str})
    df_trasl = pd.read_csv(url_trasl ,encoding=encoding,usecols=cols_tras,  sep=sep,dtype={'ID_PERSONA':int,
                                                                                           'COD_MOD_ORIGEN':str,
                                                                                           'ANEXO_ORIGEN':int,
                                                                                           'COD_MOD_DESTINO':str,                                                                                           
                                                                                           'ANEXO_DESTINO':int,
                                                                                            })

    df_trasl_origen = pd.merge(df_trasl,df_servicios[cl_s],left_on=["COD_MOD_ORIGEN","ANEXO_ORIGEN"],
                               right_on=["COD_MOD","ANEXO"],how="inner")

    df_trasl_origen.drop(columns=['COD_MOD', 'ANEXO'],inplace=True)
    df_trasl_origen.rename(columns={'ES_PUBLICO': 'ES_PUBLICO_ORIGEN'}, inplace=True)


    df_trasl_destino = pd.merge(df_trasl_origen,df_servicios[cl_s],left_on=["COD_MOD_DESTINO","ANEXO_DESTINO"],
                               right_on=["COD_MOD","ANEXO"],how="inner")

    df_trasl_destino.drop(columns=['COD_MOD', 'ANEXO'],inplace=True)
    df_trasl_destino.rename(columns={'ES_PUBLICO': 'ES_PUBLICO_DESTINO'}, inplace=True)

    df_trasl_destino['TRASLADO_A_PUBLICO'] = np.where((df_trasl_destino.ES_PUBLICO_ORIGEN==0) & 
                                                      (df_trasl_destino.ES_PUBLICO_DESTINO==1),1,0)


    df_trasl_destino = df_trasl_destino[df_trasl_destino["TRASLADO_A_PUBLICO"]==1].copy()
    
    df_agg_t  = df_trasl_destino.assign(
     TOTAL_TRASLADOS =   1
    ).groupby(['ID_PERSONA']).agg({'TOTAL_TRASLADOS':'sum'})

    df_agg_t.sort_values(by='TOTAL_TRASLADOS', ascending=False,inplace=True)
    df_agg_t.reset_index(inplace=True)
    

    return df_agg_t

def generar_kpis_traslado(df,ANIO,T_ANIOS):
    
    ID_PERSONA_SERIES = df['ID_PERSONA']
    ultimo_anio =ANIO- T_ANIOS

    num = 0
    list_df_m_tras = []
    cols_to_sum = []
    cols_to_drop = []
    for anio in range(ANIO,ultimo_anio,-1):
        if(anio<=2013):
            break
        col_name="TOTAL_TRASLADOS_T"
        if(num>0):
            posfix="_MENOS_{}".format(num)
            col_name = col_name+posfix
            cols_to_drop.append(col_name)
        cols_to_sum.append(col_name)
        df_m_t = pd.merge(ID_PERSONA_SERIES, get_traslados_por_anio(anio),left_on="ID_PERSONA",
                          right_on="ID_PERSONA", how='left')
        df_m_t.fillna({'TOTAL_TRASLADOS':0}, inplace=True)
        df_m_t.rename(columns={'TOTAL_TRASLADOS': col_name}, inplace=True)
        list_df_m_tras.append(df_m_t)
        num+=1

    df_final = reduce(lambda left,right: pd.merge(left,right,on='ID_PERSONA'), list_df_m_tras)
    df_final['TOTAL_TRASLADOS'] = df_final[cols_to_sum].sum(axis=1)
    df_final['MEAN_TRASLADOS'] = df_final[cols_to_sum].mean(axis=1)
    df_final['STD_TRASLADOS'] = df_final[cols_to_sum].std(axis=1)
    df_final.drop(cols_to_drop, axis = 1,inplace=True)    
    
    #df = pd.merge(df, df_final, left_on=["ID_PERSONA"],  right_on=["ID_PERSONA"],  how='inner')
    
    return df_final

def generar_kpis_traslado_a_publico(df,ANIO,T_ANIOS,df_servicios):
    
    ID_PERSONA_SERIES = df['ID_PERSONA']
    ultimo_anio =ANIO- T_ANIOS

    num = 0
    list_df_m_tras = []
    cols_to_sum = []
    cols_to_drop = []
    for anio in range(ANIO,ultimo_anio,-1):
        if(anio<=2013):
            break
        col_name="TOTAL_TRASLADOS_A_PUBLICO_T"
        if(num>0):
            posfix="_MENOS_{}".format(num)
            col_name = col_name+posfix
            cols_to_drop.append(col_name)
        cols_to_sum.append(col_name)
        df_m_t = pd.merge(ID_PERSONA_SERIES, get_traslados_a_publico(anio,df_servicios),left_on="ID_PERSONA",
                          right_on="ID_PERSONA", how='left')
        df_m_t.fillna({'TOTAL_TRASLADOS':0}, inplace=True)
        df_m_t.rename(columns={'TOTAL_TRASLADOS': col_name}, inplace=True)
        list_df_m_tras.append(df_m_t)
        num+=1

    df_final = reduce(lambda left,right: pd.merge(left,right,on='ID_PERSONA'), list_df_m_tras)
    df_final['TOTAL_TRASLADOS_A_PUBLICO'] = df_final[cols_to_sum].sum(axis=1)
    df_final['MEAN_TRASLADOS_A_PUBLICO'] = df_final[cols_to_sum].mean(axis=1)
    df_final['STD_TRASLADOS_A_PUBLICO'] = df_final[cols_to_sum].std(axis=1)
    df_final.drop(cols_to_drop, axis = 1,inplace=True)    
    
    #df = pd.merge(df, df_final, left_on=["ID_PERSONA"],  right_on=["ID_PERSONA"],  how='inner')
    
    return df_final


def agregar_traslado(df,df_t):
    df = pd.merge(df, df_t, left_on=["ID_PERSONA"],  right_on=["ID_PERSONA"],  how='left')
    df.fillna(0,inplace=True)
    df.fillna({'TOTAL_TRASLADOS':0 }, inplace=True)
    return df

def agregar_sisfoh(df,df_sisfoh):
    df = pd.merge(df, df_sisfoh, left_on=["NUMERO_DOCUMENTO_APOD"],
                  right_on=["PERSONA_NRO_DOC"],  how='left')

    df.fillna({'SISFOH_CSE':'OTROS' }, inplace=True)
    del df["PERSONA_NRO_DOC"]
    
    return df


def get_sisfoh():
    url_sisfoh = 'DATA_FASE_1/SISFOH/NOMINAL_SISFOH.csv'
    cols = ['PERSONA_NRO_DOC','SISFOH_CSE']    
    df_sisfoh = pd.read_csv(url_sisfoh ,usecols=cols, encoding='utf-8', dtype={'PERSONA_NRO_DOC':str})
    return df_sisfoh

def get_list_regiones():
    df = get_df_ser_risk("","")
    return df.D_REGION.unique()

def get_df_ser_risk(region,gestion=""):
    url_ser_risk = 'DATA_FASE_1/SSEE_Desercion.dta'
    df_ser_risk = pd.read_stata(url_ser_risk)
    df_ser_risk = df_ser_risk[df_ser_risk['select']==1]
    df_ser_risk.rename(columns={'cod_mod': 'COD_MOD', 'anexo': 'ANEXO', 'd_region': 'D_REGION'}, inplace=True)
    df_ser_risk['ANEXO'] = df_ser_risk['ANEXO'].astype('uint8') 
    #print(df_ser_risk.columns)
    if region!="" or gestion!="":
        if(region!=""):
            df_ser_risk = df_ser_risk[df_ser_risk.D_REGION==region].copy()
            df_ser_risk.reset_index(drop=True,inplace=True)
            
        if(gestion!=""):
            df_ser_risk = df_ser_risk[df_ser_risk.gestion==gestion].copy()
            df_ser_risk.reset_index(drop=True,inplace=True)            
                
        return df_ser_risk
    else:
        return df_ser_risk
    
    
def get_df_ser_risk_old():    
    url_ser_risk_old = 'DATA_FASE_1/Base_Desercion_5.dta'
    df_ser_risk_old = pd.read_stata(url_ser_risk_old)
    df_ser_risk_old = df_ser_risk_old[df_ser_risk_old['uid']==1]
    df_ser_risk_old.rename(columns={'cod_mod': 'COD_MOD', 'anexo': 'ANEXO', 'd_region': 'D_REGION'}, inplace=True)
    #df_ser_risk_old = df_ser_risk_old[['COD_MOD','ANEXO']].copy()
    df_ser_risk_old['ANEXO'] = df_ser_risk_old['ANEXO'].astype('uint8')
    df_ser_risk_old.reset_index(drop=True,inplace=True)    
    return df_ser_risk_old
 

def get_df_notas(anio,region):
    url_notas = 'DATA_FASE_1/notas/{}/NOTAS_POR_ALUMNO_FULL_{}.csv'.format(region,anio)
    df_notas = pd.read_csv(url_notas ,encoding='utf-8', dtype={'COD_MOD':str,'ANEXO':int})
    return df_notas

'''
def get_df_notas(anio):
    url_notas = 'DATA_FASE_1/NOTAS_POR_ALUMNO_{}.csv'.format(anio)
    df_notas = pd.read_csv(url_notas ,encoding='utf-8', dtype={'COD_MOD':str,'ANEXO':int})
    return df_notas
'''

def get_df_por_alum(df_notas_f):

    dataSet_por_alumno = df_notas_f.assign(

     #A3 A2 A5  B0 F0
     NOTA_MATEMATICA_X_ALUMNO =   np.where((df_notas_f['DSC_AREA']=='MATEMÁTICA') &
                                            (df_notas_f['NOTA_AREA_REGULAR']>=0),
                                             df_notas_f['NOTA_AREA_REGULAR'],np.NaN),   

     NOTA_COMUNICACION_X_ALUMNO =   np.where((df_notas_f['DSC_AREA']=='COMUNICACIÓN') &
                                             (df_notas_f['NOTA_AREA_REGULAR']>=0),
                                              df_notas_f['NOTA_AREA_REGULAR'],np.NaN), 
        
     NOTA_OTROS_X_ALUMNO =   np.where((df_notas_f['DSC_AREA']!='MATEMÁTICA') &
                                      (df_notas_f['DSC_AREA']!='COMUNICACIÓN') &
                                            (df_notas_f['NOTA_AREA_REGULAR']>=0),
                                             df_notas_f['NOTA_AREA_REGULAR'],np.NaN), 

    ).groupby(['COD_MOD','ANEXO','ID_PERSONA']).agg({

                                                     'NOTA_MATEMATICA_X_ALUMNO':'mean',
                                                     'NOTA_COMUNICACION_X_ALUMNO':'mean',
                                                     'NOTA_OTROS_X_ALUMNO':'mean',
                                                   })
    

    return dataSet_por_alumno



def get_df_por_grado_serv(df_notas_f):

    dataSet_por_nivel_grado = df_notas_f.assign(   

    ############## mean ##############   
     #A3 A2 A5  B0 F0
     MEAN_NOTA_MATEMATICA_X_CODMOD_NVL_GR =  np.where(df_notas_f['DSC_AREA']=='MATEMÁTICA', 
                                                       df_notas_f['NOTA_AREA_REGULAR'],np.NaN),  
        
     MEAN_NOTA_COMUNICACION_X_CODMOD_NVL_GR = np.where(df_notas_f['DSC_AREA']=='COMUNICACIÓN',
                                                       df_notas_f['NOTA_AREA_REGULAR'],np.NaN),  
        
     MEAN_NOTA_OTROS_X_CODMOD_NVL_GR =   np.where((df_notas_f['DSC_AREA']!='MATEMÁTICA') &
                                                  (df_notas_f['DSC_AREA']!='COMUNICACIÓN') &
                                                  (df_notas_f['NOTA_AREA_REGULAR']>=0),
                                                   df_notas_f['NOTA_AREA_REGULAR'],np.NaN), 

     ############## std ##############   
     #A3 A2 A5  B0 F0
     STD_NOTA_MATEMATICA_X_CODMOD_NVL_GR =   np.where(df_notas_f['DSC_AREA']=='MATEMÁTICA', 
                                                      df_notas_f['NOTA_AREA_REGULAR'],np.NaN),   
     STD_NOTA_COMUNICACION_X_CODMOD_NVL_GR =   np.where(df_notas_f['DSC_AREA']=='COMUNICACIÓN',
                                                        df_notas_f['NOTA_AREA_REGULAR'],np.NaN),  
        
     STD_NOTA_OTROS_X_CODMOD_NVL_GR =   np.where((df_notas_f['DSC_AREA']!='MATEMÁTICA') &
                                                  (df_notas_f['DSC_AREA']!='COMUNICACIÓN') &
                                                  (df_notas_f['NOTA_AREA_REGULAR']>=0),
                                                   df_notas_f['NOTA_AREA_REGULAR'],np.NaN), 


    ).groupby(['COD_MOD','ANEXO']).agg({
                                        'MEAN_NOTA_MATEMATICA_X_CODMOD_NVL_GR':'mean',
                                        'MEAN_NOTA_COMUNICACION_X_CODMOD_NVL_GR':'mean',  
                                        'MEAN_NOTA_OTROS_X_CODMOD_NVL_GR':'mean', 

                                        'STD_NOTA_MATEMATICA_X_CODMOD_NVL_GR':'std',
                                        'STD_NOTA_COMUNICACION_X_CODMOD_NVL_GR':'std',
                                        'STD_NOTA_OTROS_X_CODMOD_NVL_GR':'std',
                                       })
    
    #dataSet_por_nivel_grado['COD_MOD']=dataSet_por_nivel_grado['COD_MOD'].apply(lambda x: '{0:0>7}'.format(x))
    #dataSet_por_nivel_grado['ANEXO']=dataSet_por_nivel_grado['ANEXO'].astype('int')

    return dataSet_por_nivel_grado

def get_df_final_notas_alumn(df_notas_f):
    #print(df_notas_f.dtypes)
    df_a = get_df_por_alum(df_notas_f)
    df_a.reset_index(inplace=True)
    df_a['COD_MOD']=df_a['COD_MOD'].apply(lambda x: '{0:0>7}'.format(x))
    df_a['ANEXO']=df_a['ANEXO'].astype('int')

    #dataSet_por_alumno.head()


    dataSet_por_nivel_grado = get_df_por_grado_serv(df_notas_f)
    dataSet_por_nivel_grado.reset_index(inplace=True)
    dataSet_por_nivel_grado['COD_MOD']=dataSet_por_nivel_grado['COD_MOD'].apply(lambda x: '{0:0>7}'.format(x))
    dataSet_por_nivel_grado['ANEXO']=dataSet_por_nivel_grado['ANEXO'].astype('int')
    #dataSet_por_nivel_grado.head()
    #print(df_a.dtypes)
    #print(dataSet_por_nivel_grado.dtypes)

    
    df_a = pd.merge(df_a, dataSet_por_nivel_grado, left_on=["COD_MOD","ANEXO"],  right_on=["COD_MOD","ANEXO"],  how='inner')
    
    z_m = 'Z_NOTA_MAT_T_MENOS_1'
    z_c = 'Z_NOTA_COM_T_MENOS_1'
    z_o = 'Z_NOTA_OTROS_T_MENOS_1'
    
    n_m = 'NOTA_MATEMATICA_X_ALUMNO'
    n_c = 'NOTA_COMUNICACION_X_ALUMNO'
    n_o = 'NOTA_OTROS_X_ALUMNO'

    df_a[z_m] = (df_a[n_m] -df_a['MEAN_NOTA_MATEMATICA_X_CODMOD_NVL_GR'])/df_a['STD_NOTA_MATEMATICA_X_CODMOD_NVL_GR']
    df_a[z_c] = (df_a[n_c] - df_a['MEAN_NOTA_COMUNICACION_X_CODMOD_NVL_GR'])/df_a['STD_NOTA_COMUNICACION_X_CODMOD_NVL_GR']
    df_a[z_o] = (df_a[n_o] - df_a['MEAN_NOTA_OTROS_X_CODMOD_NVL_GR'])/df_a['STD_NOTA_OTROS_X_CODMOD_NVL_GR']
    

    #df_a.fillna(0,inplace=True)
    return df_a




def agregar_notas(df,anio,region):
   
    df_notas = get_df_notas(anio,region)
    
    df_notas_f = pd.merge(df_notas , df['ID_PERSONA'], left_on='ID_PERSONA', 
                       right_on='ID_PERSONA', how='inner')
    
    df_alum_nota = get_df_final_notas_alumn(df_notas_f)
    
    column_j_alum_not = ['COD_MOD','ANEXO','ID_PERSONA']    
    
    #print(df.dtypes)
    #print(df_alum_nota.dtypes)
    
    df_join_model = pd.merge(df , df_alum_nota, left_on=['COD_MOD_T_MENOS_1','ANEXO_T_MENOS_1','ID_PERSONA'], 
                             right_on=column_j_alum_not, how='left',suffixes=('','_x'))
    del df_join_model['COD_MOD_x']
    del df_join_model['ANEXO_x']

    z_m = 'Z_NOTA_MAT_T_MENOS_1'
    z_c = 'Z_NOTA_COM_T_MENOS_1'
    z_o = 'Z_NOTA_OTROS_T_MENOS_1'
    
    agregar_na(df_join_model,z_m)
    agregar_na(df_join_model,z_c)
    agregar_na(df_join_model,z_o)
    
    #df_join_model.fillna(0,inplace=True)
    return df_join_model



def get_df_join_train_eval(idgrado,anio_model,anio_eval,desercion_eval,short):
    delta = 1
    if short:
        columns = ["ID_PERSONA","COD_MOD","ANEXO",'COD_MOD_T_MENOS_1','ANEXO_T_MENOS_1']
    else:
        columns = None
    
    #df_sisfoh = get_sisfoh()

    deser_column_model = 'DESERCION_{}_{}'.format(anio_model,anio_model+delta)
    deser_column_eval  = 'DESERCION_{}_{}'.format(anio_eval,anio_eval+delta)
    
    edad_c_model = 'EDAD_AL_{}'.format(anio_model)
    edad_c_eval  = 'EDAD_AL_{}'.format(anio_eval)
    
    url_alumn = 'DATA_FASE_1/nominal/nominal_large_{}_{}_delta_{}.csv'.format(idgrado,anio_model,delta)
    df_alumn = pd.read_csv(url_alumn ,usecols=columns, encoding='utf-8', dtype={'COD_MOD':str,'ANEXO':int,'N_DOC':str,
                                                                 'COD_MOD_T_MENOS_1':str,'ANEXO_T_MENOS_1':int,
                                                                 'NUMERO_DOCUMENTO_APOD':str})
    df_alumn.rename(columns={deser_column_model: 'DESERCION',edad_c_model:'EDAD'}, inplace=True)
  
    

    url_alumn_eval = 'DATA_FASE_1/nominal/nominal_large_{}_{}_delta_{}.csv'.format(idgrado,anio_eval,delta)
    df_alumn_eval = pd.read_csv(url_alumn_eval,usecols=columns ,encoding='utf-8', dtype={'COD_MOD':str,'ANEXO':int,'N_DOC':str,
                                                                           'COD_MOD_T_MENOS_1':str,'ANEXO_T_MENOS_1':int,
                                                                           'NUMERO_DOCUMENTO_APOD':str})
    if desercion_eval:
        df_alumn_eval.rename(columns={deser_column_eval: 'DESERCION',edad_c_eval:'EDAD'}, inplace=True)
    else:
        df_alumn_eval.rename(columns={edad_c_eval:'EDAD'}, inplace=True)
        
    print("url_alumn : ",url_alumn," : ",df_alumn.shape)
    print("url_alumn_eval : ",url_alumn_eval," : ",df_alumn_eval.shape)
    
    return df_alumn, df_alumn_eval

def agregar_na(df,c_name):
    new_c = 'NA_'+c_name 
    df[new_c] = np.where((df[c_name].isna()) , 1 , 0 )

def load_data_correccion(idgrado,anio_model,anio_eval,delta,region,gestion,estrategia,df_sisfoh,df_servicios,df,df_eval):
    
    column_key_serv = ['COD_MOD','ANEXO']
    
    df_ser_risk = get_df_ser_risk(region,gestion)
    #df_ser_risk = get_df_ser_risk_old()
    print("before df_ser_risk: ", df_ser_risk.shape)
    df_ser_risk = pd.merge(df_ser_risk , df_servicios[column_key_serv+['ES_URBANA','ES_PUBLICO','ES_MIXTO']], left_on=column_key_serv, 
                       right_on=column_key_serv, how='inner',suffixes=('','_x'))
    print("before df_ser_risk: ", df_ser_risk.shape)
    df_alumn = df.copy()
    df_alumn_eval = df_eval.copy()

    ## join con servicios en riesgo
    
    
   

    df_alumn.ANEXO = df_alumn.ANEXO.astype(int)
        
    df_ser_risk.ANEXO = df_ser_risk.ANEXO.astype(int)

    column_forein_serv = ['D_REGION','ES_URBANA','ES_PUBLICO','ES_MIXTO']
    df_join = pd.merge(df_alumn , df_ser_risk[column_key_serv+column_forein_serv], left_on=column_key_serv, 
                       right_on=column_key_serv, how='inner',suffixes=('','_x'))
    #print(df_join.columns)

    #############################################################
    #join con notas: model

    
    anio_model_menos_1 = anio_model -1
    
    df_join_model = agregar_notas(df_join,anio_model_menos_1,region)
    
  
    ##### eval ##########
    
    
    df_join_eval = pd.merge(df_alumn_eval , df_ser_risk[column_key_serv+column_forein_serv], 
                            left_on=column_key_serv,right_on=column_key_serv, how='inner',suffixes=['','_x'])

    #join con notas: eval
    anio_eval_menos_1 = anio_eval-1
    
    df_join_new_eval = agregar_notas(df_join_eval,anio_eval_menos_1,region)
    
    
    c = ['ID_PERSONA',
         'Z_NOTA_MAT_T_MENOS_1','NA_Z_NOTA_MAT_T_MENOS_1',
         'Z_NOTA_COM_T_MENOS_1','NA_Z_NOTA_COM_T_MENOS_1',
         'Z_NOTA_OTROS_T_MENOS_1','NA_Z_NOTA_OTROS_T_MENOS_1',
         'ES_URBANA','ES_PUBLICO','ES_MIXTO'
        ]
    
    c_to_delete = ['Z_NOTA_MAT_T_MENOS_1','Z_NOTA_COM_T_MENOS_1','Z_NOTA_OTROS_T_MENOS_1']
    
    return df_join_model[c] , df_join_new_eval[c] , c_to_delete



def add_kpis_nivel_grado(ID_GRADO,df,df_eval,df_se,df_se_n_mas_1,df_sisfoh,df_servicios,
                         df_kpis_juntos,df_kpis_juntos_t_mas_1,
                         df_tras,df_tras_eval,
                         df_tras_pub,df_tras_pub_eval,
                         df_deser,df_deser_eval,
                         df_sfr,df_sfr_eval,df_dist,
                         anio_model,anio_eval):
 
    print("-----------------------------------------------------------")
    print("INICIO DE PROCESAR EL GRADO {}:",ID_GRADO)
    print("df:",df.shape)
    print("df_eval:",df_eval.shape)

    if(ID_GRADO==9):
            start = time.time()
            print("add_distancia_prim_sec:inicio")
            print("df:",df.shape)
            print("df_eval:",df_eval.shape)
            df = pd.merge(df,df_dist, left_on=['COD_MOD','ANEXO'],right_on=['COD_MOD','ANEXO'], how='left')   
            df_eval = pd.merge(df_eval , df_dist, left_on=['COD_MOD','ANEXO'], right_on=['COD_MOD','ANEXO'], how='left')            
            print("df:",df.shape)
            print("df_eval:",df_eval.shape)
            print("add_distancia_prim_sec:Time elapsed: ", time.time() - start)
    
    print("add_shock_economico:inicio")
    start = time.time()
    print("df:",df.shape)
    print("df_eval:",df_eval.shape)
    df = agregar_shock_economico(df,df_se)
    df_eval = agregar_shock_economico(df_eval,df_se_n_mas_1)
    print("df:",df.shape)
    print("df_eval:",df_eval.shape)
    print("add_shock_economico:Time elapsed: ", time.time() - start)
    
    print("add_sisfoh:inicio")
    start = time.time()
    print("df:",df.shape)
    print("df_eval:",df_eval.shape)
    df = agregar_sisfoh(df,df_sisfoh)
    df_eval = agregar_sisfoh(df_eval,df_sisfoh)
    print("df:",df.shape)
    print("df_eval:",df_eval.shape)
    print("add_sisfoh:Time elapsed: ", time.time() - start)
  
    print("add_kpis_traslado:inicio")
    start = time.time()
    print("df:",df.shape)
    print("df_eval:",df_eval.shape)
    df = add_kpis_general(df,df_tras)
    df_eval = add_kpis_general(df_eval,df_tras_eval)
    print("df:",df.shape)
    print("df_eval:",df_eval.shape)
    print("add_kpis_traslado:Time elapsed: ", time.time() - start)
    
    print("add_kpis_traslado_a_publico:inicio")
    start = time.time()
    print("df:",df.shape)
    print("df_eval:",df_eval.shape)
    df = add_kpis_general(df,df_tras_pub)
    df_eval = add_kpis_general(df_eval,df_tras_pub_eval)
    print("df:",df.shape)
    print("df_eval:",df_eval.shape)
    print("add_kpis_traslado_a_publico:Time elapsed: ", time.time() - start)   
    
    print("add_kpis_desercion:inicio")
    start = time.time()
    print("df:",df.shape)
    print("df_eval:",df_eval.shape)
    df = add_kpis_general(df,df_deser)
    df_eval = add_kpis_general(df_eval,df_deser_eval)
    print("df:",df.shape)
    print("df_eval:",df_eval.shape)
    print("add_kpis_desercion:Time elapsed: ", time.time() - start)  
    
    print("add_kpis_sfr:inicio")
    start = time.time()
    df = add_kpis_general(df,df_sfr)
    df_eval = add_kpis_general(df_eval,df_sfr_eval)
    print("add_kpis_sfr:Time elapsed: ", time.time() - start)  
           
    print("add_kpis_juntos:inicio") 
    start = time.time()
    df = add_kpis_general(df,df_kpis_juntos)
    df_eval = add_kpis_general(df_eval,df_kpis_juntos_t_mas_1)    
    print("add_kpis_juntos:Time elapsed: ", time.time() - start)  
          
    print("formateos :inicio") 
    start = time.time()
    formatear_dias_decimales(df)    
    formatear_dias_decimales(df_eval)    
    
    formatear_ES_MUJER(df)    
    formatear_ES_MUJER(df_eval)       
    
    formatear_ES_MUJER_APOD(df)    
    formatear_ES_MUJER_APOD(df_eval)  
    
    formatear_lengua_nacionalidad(df)    
    formatear_lengua_nacionalidad(df_eval)      
    
    formatear_TIENE_DNI(df)    
    formatear_TIENE_DNI(df_eval)       
    
    formatear_TIENE_DISCAPACIDAD(df)    
    formatear_TIENE_DISCAPACIDAD(df_eval)    
        
    formatear_NO_VIVE_ALGUN_PADRE(df)    
    formatear_NO_VIVE_ALGUN_PADRE(df_eval)       
    
    formatear_TIENE_PADRES_COMO_APODERADO(df)    
    formatear_TIENE_PADRES_COMO_APODERADO(df_eval)  

    formatear_TIENE_TRABAJO(df)    
    formatear_TIENE_TRABAJO(df_eval)  
    print("formateos:Time elapsed: ", time.time() - start)  
    
    print("FIN DE PROCESAR EL GRADO {}:",ID_GRADO)
    print("df:",df.shape)
    print("df_eval:",df_eval.shape)
   
    return df,df_eval




def load_data(idgrado,anio_model,anio_eval,region,df,df_eval):
    '''
    df_se = get_shock_economico(anio_model)
    df_se_n_mas_1 = get_shock_economico(anio_eval)
    '''
    df_ser_risk = get_df_ser_risk(region)
    #df_ser_risk = get_df_ser_risk_old()
    
    
    df_alumn = df.copy()
    df_alumn_eval = df_eval.copy()

    ## join con servicios en riesgo
    
    column_key_serv_left = ['COD_MOD','ANEXO']
    column_key_serv_right = ['COD_MOD','ANEXO']


    df_alumn.ANEXO = df_alumn.ANEXO.astype(int)
        
    df_ser_risk.ANEXO = df_ser_risk.ANEXO.astype(int)

    column_forein_serv = ['D_REGION']
    
    start = time.time()
    print("agregar_notas:inicio")
    df_join = pd.merge(df_alumn , df_ser_risk[column_key_serv_right+column_forein_serv], left_on=column_key_serv_left, 
                       right_on=column_key_serv_right, how='inner',suffixes=('','_x'))    
    anio_model_menos_1 = anio_model -1    
    df_join_model = agregar_notas(df_join,anio_model_menos_1,region)
    

    df_join_eval = pd.merge(df_alumn_eval , df_ser_risk[column_key_serv_right+column_forein_serv], left_on=column_key_serv_left, 
                       right_on=column_key_serv_right, how='inner',suffixes=['','_x'])
    anio_eval_menos_1 = anio_eval-1    
    df_join_new_eval = agregar_notas(df_join_eval,anio_eval_menos_1,region)
    print("agregar_notas:Time elapsed: ", time.time() - start)
      
        
    #df_join_model = reduce_mem_usage(df_join_model)  
    #df_join_new_eval = reduce_mem_usage(df_join_new_eval)  
        
    ################################################
    '''
    print("agregar_shock_economico:inicio")
    start = time.time()
    df_join_model = agregar_shock_economico(df_join_model,df_se)
    df_join_new_eval = agregar_shock_economico(df_join_new_eval,df_se_n_mas_1)
    print("agregar_shock_economico:Time elapsed: ", time.time() - start)
    
    
    print("agregar_sisfoh:inicio")
    df_join_model = agregar_sisfoh(df_join_model,df_sisfoh)
    df_join_new_eval = agregar_sisfoh(df_join_new_eval,df_sisfoh)
    print("agregar_sisfoh:Time elapsed: ", time.time() - start)
    
    print("generar_kpis_traslado:inicio")
    df_join_model = generar_kpis_traslado(df_join_model,anio_model,7)
    df_join_new_eval = generar_kpis_traslado(df_join_new_eval,anio_eval,7)
    print("generar_kpis_traslado:Time elapsed: ", time.time() - start)
    
    print("generar_kpis_traslado_a_publico:inicio")
    df_join_model = generar_kpis_traslado_a_publico(df_join_model,anio_model,7,df_servicios)
    df_join_new_eval = generar_kpis_traslado_a_publico(df_join_new_eval,anio_eval,7,df_servicios)
    print("generar_kpis_traslado_a_publico:Time elapsed: ", time.time() - start)   
    
    
    ##############################################
    df_join_model = generar_kpis_desercion(df_join_model,anio_model,7)
    df_join_new_eval = generar_kpis_desercion(df_join_new_eval,anio_eval,7)
    
    df_join_model = generar_kpis_sfr(df_join_model,anio_model,7)
    df_join_new_eval = generar_kpis_sfr(df_join_new_eval,anio_eval,7)
    
           
    df_join_model = generar_kpis_juntos(df_join_model,anio_model,7)
    df_join_new_eval = generar_kpis_juntos(df_join_new_eval,anio_eval,7)
          
    formatear_dias_decimales(df_join_model)    
    formatear_dias_decimales(df_join_new_eval)    
    
    formatear_ES_MUJER(df_join_model)    
    formatear_ES_MUJER(df_join_new_eval)       
    
    formatear_ES_MUJER_APOD(df_join_model)    
    formatear_ES_MUJER_APOD(df_join_new_eval)  
    
    #print(df_join_model.columns)
    formatear_lengua_nacionalidad(df_join_model)    
    formatear_lengua_nacionalidad(df_join_new_eval)      
    
    formatear_TIENE_DNI(df_join_model)    
    formatear_TIENE_DNI(df_join_new_eval)       
    
    formatear_TIENE_DISCAPACIDAD(df_join_model)    
    formatear_TIENE_DISCAPACIDAD(df_join_new_eval)    
        
    formatear_NO_VIVE_ALGUN_PADRE(df_join_model)    
    formatear_NO_VIVE_ALGUN_PADRE(df_join_new_eval)       
    
    formatear_TIENE_PADRES_COMO_APODERADO(df_join_model)    
    formatear_TIENE_PADRES_COMO_APODERADO(df_join_new_eval)  

    formatear_TIENE_TRABAJO(df_join_model)    
    formatear_TIENE_TRABAJO(df_join_new_eval)  
    '''
    
    return df_join_model , df_join_new_eval


def formatear_TIENE_TRABAJO(df):
    df['TIENE_TRABAJO'] = np.where((df.TRABAJA==1) | (df.HORAS_SEMANALES_TRABAJO>0) , 1 , 0 )
    df.drop(['TRABAJA','HORAS_SEMANALES_TRABAJO'], axis = 1,inplace=True) 
    

def formatear_TIENE_PADRES_COMO_APODERADO(df):
    df['TIENE_PADRES_COMO_APODERADO'] = np.where((df.PARENTESCO=="MADRE") | (df.PARENTESCO=="PADRE") , 1 , 0 )
    df.drop(['PARENTESCO'], axis = 1,inplace=True) 

def formatear_NO_VIVE_ALGUN_PADRE(df):
    df['NO_VIVE_ALGUN_PADRE'] = np.where((df.PADRE_VIVE=="NO") | (df.MADRE_VIVE=="NO") , 1 , 0 )
    df.drop(['PADRE_VIVE','MADRE_VIVE'], axis = 1,inplace=True) 

def formatear_TIENE_DISCAPACIDAD(df):
    df['TIENE_DISCAPACIDAD'] = np.where((df.DSC_DISCAPACIDAD!=0) | (df.TIENE_CERTIFICADO_DISCAPACIDAD=="SI") , 1 , 0 )
    df.drop(['DSC_DISCAPACIDAD','TIENE_CERTIFICADO_DISCAPACIDAD'], axis = 1,inplace=True) 

def formatear_TIENE_DNI(df):
    df['TIENE_DNI'] = np.where(df.N_DOC==0 , 0 , 1 )
    #df.drop(['EDAD_EN_DIAS_T','EDAD'], axis = 1,inplace=True)  

def formatear_dias_decimales(df):
    df['EDAD_EN_DECIMALES_T'] = df.EDAD_EN_DIAS_T/365.25
    df['EDAD_EN_DECIMALES_T'] = df.EDAD_EN_DECIMALES_T.round(2)
    df.drop(['EDAD_EN_DIAS_T','EDAD'], axis = 1,inplace=True)  

def formatear_ES_MUJER(df):
    df['ES_MUJER'] = np.where(df.SEXO=="MUJER" , 1 , 0 )
    df.drop(['SEXO'], axis = 1,inplace=True)  
    
def formatear_ES_MUJER_APOD(df):
    df['ES_MUJER_APOD'] = np.where(df.SEXO_APOD=="MUJER" , 1 , 0 )
    df.drop(['SEXO_APOD'], axis = 1,inplace=True) 

def formatear_lengua_nacionalidad(df):
    df['ES_LENGUA_CASTELLANA'] = np.where(df.DSC_LENGUA=="CASTELLANO" , 1 , 0 )
    df['ES_PERUANO'] = np.where(df.DSC_PAIS.str.startswith('Per', na=False) , 1 , 0 )    
    #df.drop(['DSC_LENGUA','DSC_PAIS'], axis = 1,inplace=True)  
    
    

def limpiar_SITUACION_FINAL_T_MENOS_1(df):
    df.loc[df['SITUACION_MATRICULA_T'] == 'REENTRANTE', 'SITUACION_FINAL_T_MENOS_1'] = 'RETIRADO' 
    
    df.loc[(df['SITUACION_MATRICULA_T'] == 'PROMOVIDO') & 
           (df['SITUACION_FINAL_T_MENOS_1'] == 'MATRICULADO') , 'SITUACION_FINAL_T_MENOS_1'] = 'APROBADO' 

    df.loc[(df['SITUACION_MATRICULA_T'] == 'PROMOVIDO') & 
           (df['SITUACION_FINAL_T_MENOS_1'] == 'DESAPROBADO') , 'SITUACION_FINAL_T_MENOS_1'] = 'APROBADO' 
    
    df.loc[(df['SITUACION_MATRICULA_T'] == 'PROMOVIDO') & 
           (df['SITUACION_FINAL_T_MENOS_1'] == 'SIN EVALUAR CALLAO') , 'SITUACION_FINAL_T_MENOS_1'] = 'APROBADO' 
    
        
    df.loc[(df['SITUACION_MATRICULA_T'] == 'PROMOVIDO') & 
           (df['SITUACION_FINAL_T_MENOS_1'] == 'FALLECIDO') , 'SITUACION_FINAL_T_MENOS_1'] = 'APROBADO' 
    
    df.loc[(df['SITUACION_MATRICULA_T'] == 'PROMOVIDO') & 
           (df['SITUACION_FINAL_T_MENOS_1'] == 'POSTERGACION') , 'SITUACION_FINAL_T_MENOS_1'] = 'APROBADO' 

    df.loc[(df['SITUACION_MATRICULA_T'] == 'REPITE') & 
           (df['SITUACION_FINAL_T_MENOS_1'] == 'APROBADO') , 'SITUACION_FINAL_T_MENOS_1'] = 'DESAPROBADO' 

    df.loc[(df['SITUACION_MATRICULA_T'] == 'REPITE') & 
           (df['SITUACION_FINAL_T_MENOS_1'] == 'MATRICULADO') , 'SITUACION_FINAL_T_MENOS_1'] = 'RETIRADO' 

    
    
    
    
def fe_df(df_join,df_join_eval):

    df_join.rename(columns={'COD_MOD': 'COD_MOD_T','ANEXO':'ANEXO_T'}, inplace=True)
    df_join_eval.rename(columns={'COD_MOD': 'COD_MOD_T','ANEXO':'ANEXO_T'}, inplace=True)
        
    df_join['DESERCION_T_MENOS_1__T'] = np.where((df_join['COD_MOD_T']=="000000") , 1 , 0 )
    #print("rarooooooooooooooooo")
    #print(df_join_eval.columns)
    df_join_eval['DESERCION_T_MENOS_1__T'] = np.where((df_join_eval['COD_MOD_T']=="000000") , 1 , 0 )
    
    ######  limpiar columnas ######################
    if 'SITUACION_MATRICULA_T' in df_join.columns:        
        df_join['SITUACION_MATRICULA_T'] = df_join['SITUACION_MATRICULA_T'].astype('str')
        df_join['SITUACION_MATRICULA_T'] = df_join['SITUACION_MATRICULA_T'].str.strip()
        
    if 'SITUACION_MATRICULA_T' in df_join_eval.columns:        
        df_join_eval['SITUACION_MATRICULA_T'] = df_join_eval['SITUACION_MATRICULA_T'].astype('str')
        df_join_eval['SITUACION_MATRICULA_T'] = df_join_eval['SITUACION_MATRICULA_T'].str.strip()
        
        #df_join_eval.loc[df_join_eval['SITUACION_MATRICULA'] == '0', 'SITUACION_MATRICULA'] = 'PROMOVIDO' 
        
    if 'SITUACION_MATRICULA_T_MENOS_1' in df_join.columns:        
        df_join['SITUACION_MATRICULA_T_MENOS_1'] = df_join['SITUACION_MATRICULA_T_MENOS_1'].astype('str')
        df_join['SITUACION_MATRICULA_T_MENOS_1'] = df_join['SITUACION_MATRICULA_T_MENOS_1'].str.strip()
        df_join.loc[df_join['SITUACION_MATRICULA_T_MENOS_1'] == '0', 'SITUACION_MATRICULA_T_MENOS_1'] = 'PROMOVIDO' 

    if 'SITUACION_MATRICULA_T_MENOS_1' in df_join_eval.columns:        
        df_join_eval['SITUACION_MATRICULA_T_MENOS_1'] = df_join_eval['SITUACION_MATRICULA_T_MENOS_1'].astype('str')
        df_join_eval['SITUACION_MATRICULA_T_MENOS_1'] = df_join_eval['SITUACION_MATRICULA_T_MENOS_1'].str.strip()
        df_join_eval.loc[df_join_eval['SITUACION_MATRICULA_T_MENOS_1'] == '0', 'SITUACION_MATRICULA_T_MENOS_1'] = 'PROMOVIDO' 
    
    
    limpiar_SITUACION_FINAL_T_MENOS_1(df_join)
    limpiar_SITUACION_FINAL_T_MENOS_1(df_join_eval) 
        
    if 'SF_RECUPERACION_T_MENOS_1' in df_join.columns:         
        df_join['SF_RECUPERACION_T_MENOS_1'] = df_join['SF_RECUPERACION_T_MENOS_1'].astype('str')
        df_join['SF_RECUPERACION_T_MENOS_1'] = df_join['SF_RECUPERACION_T_MENOS_1'].str.strip()       


    if 'SF_RECUPERACION_T_MENOS_1' in df_join_eval.columns:    
        df_join_eval['SF_RECUPERACION_T_MENOS_1'] = df_join_eval['SF_RECUPERACION_T_MENOS_1'].astype('str')
        df_join_eval['SF_RECUPERACION_T_MENOS_1'] = df_join_eval['SF_RECUPERACION_T_MENOS_1'].str.strip()

        
    if 'SF_RECUPERACION' in df_join.columns:         
        df_join['SF_RECUPERACION'] = df_join['SF_RECUPERACION'].astype('str')
        df_join['SF_RECUPERACION'] = df_join['SF_RECUPERACION'].str.strip()        
        
    if 'SF_RECUPERACION' in df_join_eval.columns:    
        df_join_eval['SF_RECUPERACION'] = df_join_eval['SF_RECUPERACION'].astype('str')
        df_join_eval['SF_RECUPERACION'] = df_join_eval['SF_RECUPERACION'].str.strip()        
        
    if 'NIVEL_INSTRUCCION_APOD' in df_join.columns:  
        
        df_join['NIVEL_INSTRUCCION_APOD'].fillna('OTRO', inplace=True)
        df_join_eval['NIVEL_INSTRUCCION_APOD'].fillna('OTRO', inplace=True)

        df_join['NIVEL_INSTRUCCION_APOD'] = df_join['NIVEL_INSTRUCCION_APOD'].astype('str')
        df_join['NIVEL_INSTRUCCION_APOD'] = df_join['NIVEL_INSTRUCCION_APOD'].str.strip()

        df_join_eval['NIVEL_INSTRUCCION_APOD'] = df_join_eval['NIVEL_INSTRUCCION_APOD'].astype('str')
        df_join_eval['NIVEL_INSTRUCCION_APOD'] = df_join_eval['NIVEL_INSTRUCCION_APOD'].str.strip()

        ##### crear variables derivadas ###############

        anios_esc = {'NINGUNO':0,              
                     'OTRO':0,
                     'PRIMARIA INCOMPLETA':4,
                     'PRIMARIA COMPLETA':6,
                     'SECUNDARIA INCOMPLETA':9,
                     'SECUNDARIA COMPLETA':11,
                     'SUPERIOR NO UNIV.INCOMPLETA':14,
                     'SUPERIOR NO UNIV.COMPLETA':16,
                     'SUPERIOR UNIV.INCOMPLETA':14,
                     'SUPERIOR UNIV.COMPLETA':16,
                     'SUPERIOR POST GRADUADO':20
                    }

        t_edc_apo = {'NINGUNO':'NINGUNO',              
                     'OTRO':'NINGUNO',
                     'PRIMARIA INCOMPLETA':'BASICA_INCOMPLETA',
                     'PRIMARIA COMPLETA':'BASICA_INCOMPLETA',
                     'SECUNDARIA INCOMPLETA':'BASICA_INCOMPLETA',
                     'SECUNDARIA COMPLETA':'BASICA_COMPLETA',
                     'SUPERIOR NO UNIV.INCOMPLETA':'SUPERIOR_INCOMPLETA',  
                     'SUPERIOR UNIV.INCOMPLETA':'SUPERIOR_INCOMPLETA',  
                     'SUPERIOR NO UNIV.COMPLETA':'SUPERIOR_COMPLETA',            
                     'SUPERIOR UNIV.COMPLETA':'SUPERIOR_COMPLETA',
                     'SUPERIOR POST GRADUADO':'SUPERIOR_COMPLETA',
                    }


        df_join['ANIOS_ESCOLARIDAD_APOD'] = df_join['NIVEL_INSTRUCCION_APOD'].map(anios_esc)
        df_join_eval['ANIOS_ESCOLARIDAD_APOD'] = df_join_eval['NIVEL_INSTRUCCION_APOD'].map(anios_esc)

        df_join['ANIOS_ESCOLARIDAD_APOD'].fillna(0, inplace=True)
        df_join_eval['ANIOS_ESCOLARIDAD_APOD'].fillna(0, inplace=True)
      
        
        df_join['NIVEL_INSTRUCCION_APOD_SHORT'] = df_join['NIVEL_INSTRUCCION_APOD'].map(t_edc_apo)
        df_join_eval['NIVEL_INSTRUCCION_APOD_SHORT'] = df_join_eval['NIVEL_INSTRUCCION_APOD'].map(t_edc_apo)

        df_join['NIVEL_INSTRUCCION_APOD_SHORT'] = df_join['NIVEL_INSTRUCCION_APOD_SHORT'].astype('str')
        df_join['NIVEL_INSTRUCCION_APOD_SHORT'] = df_join['NIVEL_INSTRUCCION_APOD_SHORT'].str.strip()

        df_join_eval['NIVEL_INSTRUCCION_APOD_SHORT'] = df_join_eval['NIVEL_INSTRUCCION_APOD_SHORT'].astype('str')
        df_join_eval['NIVEL_INSTRUCCION_APOD_SHORT'] = df_join_eval['NIVEL_INSTRUCCION_APOD_SHORT'].str.strip()
    

    
def get_df_servicios():
    dbf_ser = Dbf5("DATA_FASE_1/Padron_web.dbf", codec='ISO-8859-1')
    df_servicios = dbf_ser.to_dataframe()
    df_servicios = df_servicios[["COD_MOD","ANEXO","CODGEO","CODCCPP","GESTION","DAREACENSO",'D_TIPSSEXO']].copy() 
    df_servicios["ANEXO"] =df_servicios['ANEXO'].astype("int8")
    df_servicios["GESTION"] =df_servicios['GESTION'].astype("int8")    

    #df_servicios["AREA_CENSO"] =df_servicios['AREA_CENSO'].astype("int8")
    df_servicios['ES_PUBLICO'] = np.where(df_servicios['GESTION'].isin([1,2]),1,0)
    df_servicios['ES_URBANA'] = np.where(df_servicios['DAREACENSO']=='Urbana',1,0)
    df_servicios['ES_MIXTO'] = np.where(df_servicios['D_TIPSSEXO']=='Mixto',1,0)
    df_servicios['COD_MOD']=df_servicios['COD_MOD'].apply(lambda x: '{0:0>7}'.format(x))

    
    return df_servicios
'''    
def get_shock_economico(anio):
    columns_n = ["id_persona","id_hog_imp_f","log_ing_t_mas_1_imp_dist"]
    #columns_n = ["id_persona","log_ing_t_imp_dist","p_t_mas_1_hat"]    
    url = "workfile_{}.csv".format(anio)
    ds_ = pd.read_csv(url,usecols=columns_n) #
    ds_["NA_LOG_ING_T_MAS_1"] = np.where((ds_["id_hog_imp_f"].isna()) , 1 , 0 )
    ds_["LOG_ING_T_MAS_1"] = np.where((ds_["id_hog_imp_f"].isna()) , ds_["id_hog_imp_f"] , ds_["log_ing_t_mas_1_imp_dist"] )
    del ds_["id_hog_imp_f"]
    ds_.columns = [x.upper() for x in ds_.columns]
    return ds_
'''
def get_shock_economico(anio,nivel="EBR"):
    columns_n = ["ID_PERSONA","LOG_ING_T_MAS_1_IMP_DIST","NA_LOG_ING_T_MAS_1_IMP_DIST"]   
    url = "workfile_{}_{}_v2.csv".format(nivel,anio)
    ds_ = pd.read_csv(url,usecols=columns_n) #
    return ds_


def agregar_shock_economico(df,df_se):
    df = pd.merge(df, df_se, left_on=["ID_PERSONA"],
                  right_on=["ID_PERSONA"],  how='left')

    #df.fillna({'LOG_ING_T_MAS_1_IMP_DIST':0 }, inplace=True)

    return df


def get_distancia_prim_sec():
    
    df_sec_cerca =pd.read_csv("SecundariaCerca.csv", encoding="utf-8",index_col=0) 

    df_sec_cerca.loc[(df_sec_cerca['Distancia'] == 0), 'GRUPO_DISTANCIA'] = '0K'
    df_sec_cerca.loc[(df_sec_cerca['Distancia'] > 0) & (df_sec_cerca['Distancia'] <= 1000), 'GRUPO_DISTANCIA'] = 'MENOR_1K'
    df_sec_cerca.loc[(df_sec_cerca['Distancia'] > 1000) & (df_sec_cerca['Distancia'] <= 5000), 'GRUPO_DISTANCIA'] = '1K_5K'
    df_sec_cerca.loc[(df_sec_cerca['Distancia'] > 5000), 'GRUPO_DISTANCIA'] = 'MAYOR_5K'

    df_sec_cerca.columns = [x.upper() for x in df_sec_cerca.columns]

    df_sec_cerca[['COD_MOD','ANEXO']] = df_sec_cerca.CODIGOLUGAR.str.split("-",expand=True)
    df_sec_cerca['ANEXO'] = df_sec_cerca['ANEXO'].astype('uint8')

    df_sec_cerca['COD_MOD']=df_sec_cerca['COD_MOD'].apply(lambda x: '{0:0>7}'.format(x))
    del df_sec_cerca['SECUNDARIACERCA']
    del df_sec_cerca['CODIGOLUGAR']
    
    
    return df_sec_cerca

def get_hermanos():
    df_hermanos =pd.read_csv("hermanos.csv", encoding="utf-8") 
    return df_hermanos


def incorporar_campos_corregidos(lista_regiones,lista_grados):

    ANIO_MODEL = 2019
    ANIO_EVAL  = 2020 #test real
    
    desercion_eval = False
    delta = 1
    estrategia = 5
    df_sisfoh = None

    df_se = get_shock_economico(ANIO_MODEL)
    df_se2020 = get_shock_economico(ANIO_EVAL)
    df_servicios = get_df_servicios()
    
    #df_dist = get_distancia_prim_sec()
    
    #df_hermanos = get_hermanos()

    for ID_GRADO in lista_grados:   
        
        df , df_eval = get_df_join_train_eval(ID_GRADO,ANIO_MODEL,ANIO_EVAL,delta,desercion_eval,True)

        for region in lista_regiones:
            gc.collect()
            print(region)
            start = time.time()

            url_dir = "{}/{}/".format(region,ID_GRADO)
            
            df_join , df_join_eval , c_to_delete = load_data_correccion(ID_GRADO,ANIO_MODEL,ANIO_EVAL,
                                                                        delta,region,"", estrategia,
                                                                        df_sisfoh,df_servicios,df,df_eval)
            
  
            df_join_sv , df_join_eval_sv = get_saved_join_data(url_dir,sub_dir="out")
            df_join_sv.drop(c_to_delete, axis = 1,inplace=True) 
            df_join_eval_sv.drop(c_to_delete, axis = 1,inplace=True) 

            df_join_f = pd.merge(df_join_sv , df_join, left_on=['ID_PERSONA'],   
                                 right_on=['ID_PERSONA'], how='inner',suffixes=('','_x'))
            
            df_join_eval_f = pd.merge(df_join_eval_sv , df_join_eval, left_on=['ID_PERSONA'],   
                                      right_on=['ID_PERSONA'], how='inner',suffixes=('','_x'))
            
            '''
            print("variables hermanos : ")
            
            print("before df_join_f: ",df_join_f.shape)
            df_join_f = pd.merge(df_join_f , df_hermanos, left_on=['ID_PERSONA'],   
                                 right_on=['ID_PERSONA'], how='left',suffixes=('','_x'))
            print("after df_join_f: ",df_join_f.shape)
            '''
            
            print("variables economicas : ")
            
            print("before df_join_f: ",df_join_f.shape)
            df_join_f = pd.merge(df_join_f , df_se, left_on=['ID_PERSONA'],   
                                 right_on=['ID_PERSONA'], how='inner',suffixes=('','_x'))
            print("after df_join_f: ",df_join_f.shape)
            
            print("before df_join_eval_f: ",df_join_eval_f.shape)
            df_join_eval_f = pd.merge(df_join_eval_f , df_se, left_on=['ID_PERSONA'],   
                                      right_on=['ID_PERSONA'], how='inner',suffixes=('','_x'))
            print("after df_join_eval_f: ",df_join_eval_f.shape)
            
            '''
            print("variables de distancia : ")            
          
            print("before df_join_f: ",df_join_f.shape)
            df_join_f = pd.merge(df_join_f , df_dist, left_on=['COD_MOD','ANEXO'],   
                                 right_on=['COD_MOD','ANEXO'], how='inner',suffixes=('','_x'))
            print("after df_join_f: ",df_join_f.shape)
            
            print("before df_join_eval_f: ",df_join_eval_f.shape)
            df_join_eval_f = pd.merge(df_join_eval_f , df_dist, left_on=['COD_MOD','ANEXO'],   
                                      right_on=['COD_MOD','ANEXO'], how='inner',suffixes=('','_x'))
            print("after df_join_eval_f: ",df_join_eval_f.shape)
            '''
            
            #exportar data 
            exportar_data_innominada(df_join_f,df_join_eval_f,url_dir)
            
def get_saved_join_data(url_dir,sub_dir="data"):
    
    if not url_dir:
        url_dir="reporte_modelo/"+sub_dir+"/"
    else:
        url_dir = '{}/{}'.format("reporte_modelo/"+sub_dir,url_dir)
        if not os.path.exists(url_dir):
            os.makedirs(url_dir)
        print("reporte generado en : "+url_dir)
    
    specific_url = url_dir+"data.csv"
    specific_url_eval = url_dir+"data_eval.csv"
    
    dt = {'COD_MOD':str,'COD_MOD_T':str,'ANEXO':int,'ANEXO_T':int,'EDAD':int,
          'N_DOC':str,'COD_MOD_T_MENOS_1':str,
          'ANEXO_T_MENOS_1':int,'NUMERO_DOCUMENTO_APOD':str,'ID_PERSONA':int}

    df=pd.read_csv(specific_url,dtype=dt, encoding="utf-8") 
    df_eval=pd.read_csv(specific_url_eval,dtype=dt, encoding="utf-8") 
    
    return df,df_eval


def exportar_data_innominada(df_join,df_join_eval,url_dir):
   
    #df_join["ID_PERSONA"] = np.random.randint(1,100, df_join.shape[0])
    #df_join_eval["ID_PERSONA"] = np.random.randint(1,100, df_join_eval.shape[0])

    #df_join["N_DOC"] = np.random.randint(1,100, df_join.shape[0])
    #df_join_eval["N_DOC"] = np.random.randint(1,100, df_join_eval.shape[0])

    #df_join["NUMERO_DOCUMENTO_APOD"] = np.random.randint(1,100, df_join.shape[0])
    #df_join_eval["NUMERO_DOCUMENTO_APOD"] = np.random.randint(1,100, df_join_eval.shape[0])
     
    if not url_dir:
        url_dir="reporte_modelo/data/"  
    else:

        url_dir = '{}/{}'.format("reporte_modelo/data",url_dir)
        if not os.path.exists(url_dir): 
            os.makedirs(url_dir)
        #print("reporte generado en : "+url_dir) 
    
    specific_url = url_dir+"data.csv"
    specific_url_eval = url_dir+"data_eval.csv"
    print("reporte generado en : "+specific_url)
    print("reporte generado en : "+specific_url_eval)
    #df_result_nom_geo[cls_export].to_excel(specific_url,index=False) 
    df_join.to_csv(specific_url,index =False , encoding="utf-8")
    df_join_eval.to_csv(specific_url_eval,index =False , encoding="utf-8")